@@ -36,12 +36,13 @@ filter_re_eval_string(FilterExprNode *s, LogMessage *msg, gint value_handle, con
 
   if (str_len < 0)
     str_len = strlen(str);
-  result = log_matcher_match(self->matcher, msg, value_handle, str, str_len) ^ self->super.comp;
-  msg_debug("Filter regexp node evaluation result",
-            evt_tag_printf("msg", "%p", msg),
+  result = log_matcher_match(self->matcher, msg, value_handle, str, str_len);
+  msg_trace("match() evaluation started",
             evt_tag_str("input", str),
-            evt_tag_str("result", result ? "match" : "not-match"));
-  return result;
+            evt_tag_str("pattern", self->matcher->pattern),
+            evt_tag_str("value", log_msg_get_value_name(value_handle, NULL)),
+            evt_tag_printf("msg", "%p", msg));
+  return result ^ s->comp;
 }
 
 static gboolean
@@ -67,17 +68,19 @@ filter_re_free(FilterExprNode *s)
   log_matcher_options_destroy(&self->matcher_options);
 }
 
-static void
+static gboolean
 filter_re_init(FilterExprNode *s, GlobalConfig *cfg)
 {
   FilterRE *self = (FilterRE *) s;
 
   if (self->matcher_options.flags & LMF_STORE_MATCHES)
     self->super.modify = TRUE;
+
+  return TRUE;
 }
 
 gboolean
-filter_re_compile_pattern(FilterRE *self, GlobalConfig *cfg, gchar *re, GError **error)
+filter_re_compile_pattern(FilterRE *self, GlobalConfig *cfg, const gchar *re, GError **error)
 {
   log_matcher_options_init(&self->matcher_options, cfg);
   self->matcher = log_matcher_new(cfg, &self->matcher_options);
