@@ -37,206 +37,30 @@ _shall_set_values_indirectly(NVHandle value_handle)
          !log_msg_is_handle_match(value_handle);
 }
 
+static void
+log_matcher_store_pattern(LogMatcher *self, const gchar *pattern)
+{
+  g_free(self->pattern);
+  self->pattern = g_strdup(pattern);
+}
+
+static void
+log_matcher_free_method(LogMatcher *self)
+{
+  g_free(self->pattern);
+}
+
 static void
 log_matcher_init(LogMatcher *self, const LogMatcherOptions *options)
 {
   self->ref_cnt = 1;
   self->flags = options->flags;
-}
-
-typedef struct _LogMatcherPosixRe
-{
-  LogMatcher super;
-  regex_t pattern;
-} LogMatcherPosixRe;
-
-static gboolean
-log_matcher_posix_re_compile(LogMatcher *s, const gchar *re, GError **error)
-{
-  LogMatcherPosixRe *self = (LogMatcherPosixRe *) s;
-  gint rc;
-  const gchar *re_comp = re;
-  gint flags = REG_EXTENDED;
-
-  g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
-
-  if (re[0] == '(' && re[1] == '?')
-    {
-      gint i;
-
-      for (i = 2; re[i] && re[i] != ')'; i++)
-        {
-          if (re[i] == 'i')
-            {
-              /* deprecated */
-              msg_warning_once("WARNING: Your configuration file uses an obsoleted regexp option, please update your configuration",
-                               evt_tag_str("option", "(?i)"),
-                               evt_tag_str("change", "use ignore-case flag instead of (?i)"));
-
-              flags |= REG_ICASE;
-            }
-        }
-      if (re[i])
-        {
-          re_comp = &re[i + 1];
-        }
-      else
-        {
-          g_set_error(error, LOG_MATCHER_ERROR, 0, "missing closing parentheses in regexp flags");
-          return FALSE;
-        }
-    }
-
-  if (self->super.flags & LMF_ICASE)
-    flags |= REG_ICASE;
-  if (self->super.flags & LMF_NEWLINE)
-    flags |= REG_NEWLINE;
-  if ((self->super.flags & (LMF_MATCH_ONLY + LMF_STORE_MATCHES)) == LMF_MATCH_ONLY)
-    flags |= REG_NOSUB;
-
-  rc = regcomp(&self->pattern, re_comp, flags);
-  if (rc)
-    {
-      gchar buf[256];
-
-      regerror(rc, &self->pattern, buf, sizeof(buf));
-      g_set_error(error, LOG_MATCHER_ERROR, 0, "Error compiling regular expression: %s", buf);
-      return FALSE;
-    }
-  return TRUE;
-}
-
-static void
-log_matcher_posix_re_feed_backrefs(LogMatcher *s, LogMessage *msg, gint value_handle, regmatch_t *matches,
-                                   const gchar *value)
-{
-  gint i;
-  gboolean indirect = _shall_set_values_indirectly(value_handle);
-
-  for (i = 0; i < RE_MAX_MATCHES && matches[i].rm_so != -1; i++)
-    {
-      if (indirect)
-        {
-          log_msg_set_match_indirect(msg, i, value_handle, 0, matches[i].rm_so, matches[i].rm_eo - matches[i].rm_so);
-        }
-      else
-        {
-          log_msg_set_match(msg, i, &value[matches[i].rm_so], matches[i].rm_eo - matches[i].rm_so);
-        }
-    }
-}
-
-static gboolean
-log_matcher_posix_re_match(LogMatcher *s, LogMessage *msg, gint value_handle, const gchar *value, gssize value_len)
-{
-  LogMatcherPosixRe *self = (LogMatcherPosixRe *) s;
-  regmatch_t matches[RE_MAX_MATCHES];
-  gboolean rc;
-  const gchar *buf;
-
-  APPEND_ZERO(buf, value, value_len);
-  rc = !regexec(&self->pattern, buf, RE_MAX_MATCHES, matches, 0);
-  if (rc && (s->flags & LMF_STORE_MATCHES))
-    {
-      log_matcher_posix_re_feed_backrefs(s, msg, value_handle, matches, value);
-    }
-  return rc;
-}
-
-static gchar *
-log_matcher_posix_re_replace(LogMatcher *s, LogMessage *msg, gint value_handle, const gchar *value, gssize value_len,
-                             LogTemplate *replacement, gssize *new_length)
-{
-  LogMatcherPosixRe *self = (LogMatcherPosixRe *) s;
-  regmatch_t matches[RE_MAX_MATCHES];
-  gboolean rc;
-  GString *new_value = NULL;
-  gsize current_ofs = 0;
-  gboolean first_round = TRUE;
-  gchar *buf;
-
-  APPEND_ZERO(buf, value, value_len);
-
-  do
-    {
-      if (current_ofs == value_len)
-        break;
-
-      rc = !regexec(&self->pattern, buf + current_ofs, RE_MAX_MATCHES, matches, current_ofs > 0 ? REG_NOTBOL : 0);
-      if (rc)
-        {
-          /* start_ofs & end_ofs are relative to the original string */
-          gsize start_ofs = matches[0].rm_so + current_ofs;
-          gsize end_ofs = matches[0].rm_eo + current_ofs;
-
-          if (start_ofs == end_ofs && !first_round)
-            {
-              start_ofs++;
-              end_ofs++;
-            }
-
-          log_matcher_posix_re_feed_backrefs(s, msg, value_handle, matches, buf + current_ofs);
-
-          if (!new_value)
-            new_value = g_string_sized_new(value_len);
-
-          g_string_append_len(new_value, buf + current_ofs, start_ofs - current_ofs);
-          log_template_append_format(replacement, msg, NULL, LTZ_LOCAL, 0, NULL, new_value);
-          current_ofs = end_ofs;
-
-          if ((self->super.flags & LMF_GLOBAL) == 0)
-            {
-              g_string_append_len(new_value, buf + current_ofs, value_len - current_ofs);
-              break;
-            }
-        }
-      else
-        {
-          if (new_value)
-            {
-              /* no more matches, append the end of the string */
-              g_string_append_len(new_value, buf + current_ofs, value_len - current_ofs);
-            }
-        }
-      first_round = FALSE;
-    }
-  while (rc && (self->super.flags & LMF_GLOBAL));
-
-  if (new_value)
-    {
-      if (new_length)
-        *new_length = new_value->len;
-      return g_string_free(new_value, FALSE);
-    }
-  return NULL;
-}
-
-static void
-log_matcher_posix_re_free(LogMatcher *s)
-{
-  LogMatcherPosixRe *self = (LogMatcherPosixRe *) s;
-
-  regfree(&self->pattern);
-}
-
-LogMatcher *
-log_matcher_posix_re_new(GlobalConfig *cfg, const LogMatcherOptions *options)
-{
-  LogMatcherPosixRe *self = g_new0(LogMatcherPosixRe, 1);
-
-  log_matcher_init(&self->super, options);
-  self->super.compile = log_matcher_posix_re_compile;
-  self->super.match = log_matcher_posix_re_match;
-  self->super.replace = log_matcher_posix_re_replace;
-  self->super.free_fn = log_matcher_posix_re_free;
-
-  return &self->super;
+  self->free_fn = log_matcher_free_method;
 }
 
 typedef struct _LogMatcherString
 {
   LogMatcher super;
-  gchar *pattern;
   gint pattern_len;
 } LogMatcherString;
 
@@ -246,9 +70,9 @@ log_matcher_string_compile(LogMatcher *s, const gchar *pattern, GError **error)
   LogMatcherString *self = (LogMatcherString *) s;
 
   g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+  log_matcher_store_pattern(s, pattern);
 
-  self->pattern = g_strdup(pattern);
-  self->pattern_len = strlen(self->pattern);
+  self->pattern_len = strlen(pattern);
   return TRUE;
 }
 
@@ -257,22 +81,23 @@ log_matcher_string_match_string(LogMatcherString *self, const gchar *value, gsiz
 {
   const gchar *result = NULL;
   gboolean match = FALSE;
+  const gchar *pattern = self->super.pattern;
 
   if (self->pattern_len > value_len)
     return NULL;
   if (G_LIKELY((self->super.flags & (LMF_SUBSTRING + LMF_PREFIX)) == 0))
     {
       if (self->super.flags & LMF_ICASE)
-        match = strncasecmp(value, self->pattern, value_len) == 0;
+        match = strncasecmp(value, pattern, value_len) == 0;
       else
-        match = strncmp(value, self->pattern, value_len) == 0;
+        match = strncmp(value, pattern, value_len) == 0;
     }
   else if (self->super.flags & LMF_PREFIX)
     {
       if (self->super.flags & LMF_ICASE)
-        match = strncasecmp(value, self->pattern, MIN(value_len, self->pattern_len)) == 0;
+        match = strncasecmp(value, pattern, MIN(value_len, self->pattern_len)) == 0;
       else
-        match = strncmp(value, self->pattern, MIN(value_len, self->pattern_len)) == 0;
+        match = strncmp(value, pattern, MIN(value_len, self->pattern_len)) == 0;
     }
   else if (self->super.flags & LMF_SUBSTRING)
     {
@@ -282,13 +107,13 @@ log_matcher_string_match_string(LogMatcherString *self, const gchar *value, gsiz
           gchar *res;
 
           APPEND_ZERO(buf, value, value_len);
-          res = strcasestr(buf, self->pattern);
+          res = strcasestr(buf, pattern);
           if (res)
             result = value + (res - buf);
         }
       else
         {
-          result = g_strstr_len(value, value_len, self->pattern);
+          result = g_strstr_len(value, value_len, pattern);
         }
     }
 
@@ -375,14 +200,6 @@ log_matcher_string_replace(LogMatcher *s, LogMessage *msg, gint value_handle, co
   return NULL;
 }
 
-static void
-log_matcher_string_free(LogMatcher *s)
-{
-  LogMatcherString *self = (LogMatcherString *) s;
-
-  g_free(self->pattern);
-}
-
 LogMatcher *
 log_matcher_string_new(GlobalConfig *cfg, const LogMatcherOptions *options)
 {
@@ -392,7 +209,6 @@ log_matcher_string_new(GlobalConfig *cfg, const LogMatcherOptions *options)
   self->super.compile = log_matcher_string_compile;
   self->super.match = log_matcher_string_match;
   self->super.replace = log_matcher_string_replace;
-  self->super.free_fn = log_matcher_string_free;
 
   return &self->super;
 }
@@ -409,6 +225,7 @@ log_matcher_glob_compile(LogMatcher *s, const gchar *pattern, GError **error)
   LogMatcherGlob *self = (LogMatcherGlob *)s;
 
   g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+  log_matcher_store_pattern(s, pattern);
 
   self->pattern = g_pattern_spec_new(pattern);
   return TRUE;
@@ -449,6 +266,7 @@ log_matcher_glob_free(LogMatcher *s)
 {
   LogMatcherGlob *self = (LogMatcherGlob *)s;
   g_pattern_spec_free(self->pattern);
+  log_matcher_free_method(s);
 }
 
 LogMatcher *
@@ -486,6 +304,7 @@ log_matcher_pcre_re_compile(LogMatcher *s, const gchar *re, GError **error)
   gint flags = 0;
 
   g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+  log_matcher_store_pattern(s, re);
 
   if (self->super.flags & LMF_ICASE)
     flags |= PCRE_CASELESS;
@@ -547,13 +366,19 @@ log_matcher_pcre_re_feed_backrefs(LogMatcher *s, LogMessage *msg, gint value_han
 
   for (i = 0; i < (RE_MAX_MATCHES) && i < match_num; i++)
     {
+      gint begin_index = matches[2 * i];
+      gint end_index = matches[2 * i + 1];
+
+      if (begin_index < 0 || end_index < 0)
+        continue;
+
       if (indirect)
         {
-          log_msg_set_match_indirect(msg, i, value_handle, 0, matches[2 * i], matches[2 * i + 1] - matches[2 * i]);
+          log_msg_set_match_indirect(msg, i, value_handle, 0, begin_index, end_index - begin_index);
         }
       else
         {
-          log_msg_set_match(msg, i, &value[matches[2 * i]], matches[2 * i + 1] - matches[2 * i]);
+          log_msg_set_match(msg, i, &value[begin_index], end_index - begin_index);
         }
     }
 }
@@ -583,7 +408,13 @@ log_matcher_pcre_re_feed_named_substrings(LogMatcher *s, LogMessage *msg, int *m
       for (i = 0; i < namecount; i++)
         {
           int n = (tabptr[0] << 8) | tabptr[1];
-          log_msg_set_value_by_name(msg, tabptr + 2, value + matches[2*n], matches[2*n+1] - matches[2*n]);
+          gint begin_index = matches[2 * n];
+          gint end_index = matches[2 * n + 1];
+
+          if (begin_index < 0 || end_index < 0)
+            continue;
+
+          log_msg_set_value_by_name(msg, tabptr + 2, value + begin_index, end_index - begin_index);
           tabptr += name_entry_size;
         }
     }
@@ -675,7 +506,7 @@ log_matcher_pcre_re_replace(LogMatcher *s, LogMessage *msg, gint value_handle, c
   last_match_was_empty = FALSE;
   do
     {
-      /* loop over the string, replacing one occurence at a time. */
+      /* loop over the string, replacing one occurrence at a time. */
 
       /* NOTE: zero length matches need special care, as we could spin
        * forever otherwise (since the current position wouldn't be
@@ -778,6 +609,7 @@ log_matcher_pcre_re_free(LogMatcher *s)
   LogMatcherPcreRe *self = (LogMatcherPcreRe *) s;
   pcre_free_study(self->extra);
   pcre_free(self->pattern);
+  log_matcher_free_method(s);
 }
 
 LogMatcher *
@@ -803,7 +635,6 @@ struct
 } matcher_types[] =
 {
   { "pcre", log_matcher_pcre_re_new },
-  { "posix", log_matcher_posix_re_new },
   { "string", log_matcher_string_new },
   { "glob", log_matcher_glob_new },
   { NULL, NULL },
@@ -854,6 +685,15 @@ log_matcher_options_set_type(LogMatcherOptions *options, const gchar *type)
 {
   LogMatcherConstructFunc construct;
 
+  if (strcmp(type, "posix") == 0)
+    {
+      msg_warning_once("WARNING: syslog-ng dropped support for POSIX regexp implementations in " VERSION_3_14
+                       " in favour of PCRE, which should be upward compatible. All 'posix' regexps are "
+                       "automatically switched to 'pcre'. Please ensure that your regexps work with PCRE and "
+                       "specify type('pcre') explicitly or increase @version to remove this warning");
+      type = "pcre";
+    }
+
   construct = log_matcher_lookup_construct(type);
   if (!construct)
     return FALSE;
@@ -903,8 +743,6 @@ log_matcher_options_init(LogMatcherOptions *options, GlobalConfig *cfg)
 
       if (cfg_is_config_version_older(cfg, 0x0306))
         {
-          msg_warning_once("WARNING: syslog-ng changed the default regexp implementation to PCRE starting from " VERSION_3_6
-                           ", please ensure your regexp works with PCRE or please specify type(\"posix\") in filters explicitly");
           default_matcher = "posix";
         }
       if (!log_matcher_options_set_type(options, default_matcher))
