@@ -83,19 +83,18 @@ log_proto_get_char_size_for_fixed_encoding(const gchar *encoding)
 }
 
 
-static gboolean
-log_proto_text_server_prepare(LogProtoServer *s, GIOCondition *cond)
+static LogProtoPrepareAction
+log_proto_text_server_prepare(LogProtoServer *s, GIOCondition *cond, gint *timeout)
 {
   LogProtoTextServer *self = (LogProtoTextServer *) s;
   gboolean avail;
 
-  if (log_proto_buffered_server_prepare(s, cond))
-    {
-      return TRUE;
-    }
+  LogProtoPrepareAction action = log_proto_buffered_server_prepare(s, cond, timeout);
+  if (action != LPPA_POLL_IO)
+    return action;
 
   avail = (self->cached_eol_pos != 0);
-  return avail;
+  return avail ? LPPA_FORCE_SCHEDULE_FETCH : LPPA_POLL_IO;
 }
 
 static void
