@@ -84,12 +84,12 @@ _allocate_thread_id(void)
     {
       for (id = 0; id < MAIN_LOOP_MAX_WORKER_THREADS; id++)
         {
-          if ((main_loop_workers_idmap[main_loop_worker_type] & (1 << id)) == 0)
+          if ((main_loop_workers_idmap[main_loop_worker_type] & (1ULL << id)) == 0)
             {
               /* id not yet used */
 
               main_loop_worker_id = (id + 1)  + (main_loop_worker_type * MAIN_LOOP_MAX_WORKER_THREADS);
-              main_loop_workers_idmap[main_loop_worker_type] |= (1 << id);
+              main_loop_workers_idmap[main_loop_worker_type] |= (1ULL << id);
               break;
             }
         }
@@ -103,7 +103,8 @@ _release_thread_id(void)
   g_static_mutex_lock(&main_loop_workers_idmap_lock);
   if (main_loop_worker_id)
     {
-      main_loop_workers_idmap[main_loop_worker_type] &= ~(1 << (main_loop_worker_id - 1));
+      const gint id = main_loop_worker_id & (sizeof(guint64) * CHAR_BIT - 1);
+      main_loop_workers_idmap[main_loop_worker_type] &= ~(1ULL << (id - 1));
       main_loop_worker_id = 0;
     }
   g_static_mutex_unlock(&main_loop_workers_idmap_lock);
@@ -177,9 +178,9 @@ main_loop_worker_thread_start(void *cookie)
   _allocate_thread_id();
   INIT_IV_LIST_HEAD(&batch_callbacks);
 
-  g_mutex_lock(&workers_running_lock);
+  g_static_mutex_lock(&workers_running_lock);
   main_loop_workers_running++;
-  g_mutex_unlock(&workers_running_lock);
+  g_static_mutex_unlock(&workers_running_lock);
 
   app_thread_start();
 }
@@ -191,11 +192,10 @@ main_loop_worker_thread_stop(void)
   app_thread_stop();
   _release_thread_id();
 
-  g_mutex_lock(&workers_running_lock);
+  g_static_mutex_lock(&workers_running_lock);
   main_loop_workers_running--;
-  g_mutex_unlock(&workers_running_lock);
-
-  g_cond_signal(&thread_halt_cond);
+  g_cond_signal(thread_halt_cond);
+  g_static_mutex_unlock(&workers_running_lock);
 }
 
 void
@@ -386,7 +386,6 @@ _reenable_worker_jobs(void *s)
 void
 main_loop_worker_sync_call(void (*func)(gpointer user_data), gpointer user_data)
 {
-
   _register_sync_call_action(&sync_call_actions, func, user_data);
 
   if (main_loop_jobs_running == 0)
@@ -400,6 +399,35 @@ main_loop_worker_sync_call(void (*func)(gpointer user_data), gpointer user_data)
     }
 }
 
+/* This function is intended to be used from test programs to properly
+ * synchronize threaded worker startups and then trigger everything to exit
+ * and wait for that too.  This is useful in LogThreadedDestDriver test
+ * cases, where the test program itself is the "main" thread and we don't
+ * want to launch an entire main loop, because in that case we'd be forced
+ * to feed the worker thread from ivykis callbacks, which is a lot more
+ * difficult to write/maintain.
+ *
+ * This function clearly shows the level of ugly couplings between the
+ * various mainloop components.  (e.g.  mainloop and mainloop worker).  I
+ * consider that this should be part of the mainloop layer (semantically, it
+ * is the main loop that we are launching in a special mode.  This is also
+ * indicated by the iv_main() call below). However its implementation
+ * requires access to the main_loop_workers variable.
+ */
+void
+main_loop_sync_worker_startup_and_teardown(void)
+{
+  struct iv_task request_exit;
+  if (main_loop_jobs_running == 0)
+    return;
+
+  IV_TASK_INIT(&request_exit);
+  request_exit.handler = (void (*)(void *)) _request_all_threads_to_exit;
+  iv_task_register(&request_exit);
+  _register_sync_call_action(&sync_call_actions, (void (*)(gpointer user_data)) iv_quit, NULL);
+  iv_main();
+}
+
 void
 main_loop_worker_init(void)
 {
