@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014 Balabit
+ * Copyright (c) 2018 Balabit
  * Copyright (c) 2014 Balázs Scheidler <bazsi@balabit.hu>
  * Copyright (c) 2014 Viktor Tusa <tusa@balabit.hu>
  *
@@ -24,60 +24,49 @@
  */
 
 #include "str-format.h"
-#include "testutils.h"
+#include <criterion/criterion.h>
 
-void test_format_hex_string__single_byte__perfect(void)
+Test(test_str_format, hex_string__single_byte__perfect)
 {
   gchar expected_output[3] = "40";
   gchar output[3];
   gchar input[1] = "@";
-  //Act
+
   format_hex_string(input, sizeof(input), output, sizeof(output));
-  //Assert
-  assert_nstring(output, sizeof(output), expected_output, sizeof(expected_output),
-                 "format_hex_string output does not match!", NULL);
+
+  cr_expect_str_eq(output, expected_output, "format_hex_string output does not match!");
 }
 
-void test_format_hex_string__two_bytes__perfect(void)
+Test(test_str_format, hex_string__two_bytes__perfect)
 {
   gchar expected_output[5] = "4041";
   gchar output[5];
   gchar input[2] = "@A";
-  //Act
+
   format_hex_string(input, sizeof(input), output, sizeof(output));
-  //Assert
-  assert_nstring(output, sizeof(output), expected_output, sizeof(expected_output),
-                 "format_hex_string output does not match with two bytes!", NULL);
+
+  cr_expect_str_eq(output, expected_output, "format_hex_string output does not match with two bytes!");
 }
 
-void test_format_hex_string_with_delimiter__single_byte__perfect(void)
+Test(test_str_format, hex_string_with_delimiter__single_byte__perfect)
 {
   gchar expected_output[3] = "40";
   gchar output[3];
   gchar input[1] = "@";
-  //Act
+
   format_hex_string_with_delimiter(input, sizeof(input), output, sizeof(output), ' ');
-  //Assert
-  assert_nstring(output, sizeof(output), expected_output, sizeof(expected_output),
-                 "format_hex_string_with_delimiter output does not match!", NULL);
+
+  cr_expect_str_eq(output, expected_output, "format_hex_string_with_delimiter output does not match!");
 }
 
-void test_format_hex_string_with_delimiter__two_bytes__perfect(void)
+Test(test_str_format, hex_string_with_delimiter__two_bytes__perfect)
 {
   gchar expected_output[6] = "40 41";
   gchar output[6];
   gchar input[2] = "@A";
-  //Act
-  format_hex_string_with_delimiter(input, sizeof(input), output, sizeof(output), ' ');
-  //Assert
-  assert_nstring(output, sizeof(output), expected_output, sizeof(expected_output),
-                 "format_hex_string_with_delimiter output does not match in case of two bytes!", NULL);
-}
 
-int main(void)
-{
-  test_format_hex_string__single_byte__perfect();
-  test_format_hex_string__two_bytes__perfect();
-  test_format_hex_string_with_delimiter__single_byte__perfect();
-  test_format_hex_string_with_delimiter__two_bytes__perfect();
+  format_hex_string_with_delimiter(input, sizeof(input), output, sizeof(output), ' ');
+
+  cr_expect_str_eq(output, expected_output,
+                   "format_hex_string_with_delimiter output does not match in case of two bytes!");
 }
