@@ -96,8 +96,8 @@ enum
 
 typedef struct
 {
-  gchar *name;
-  gchar *alt_name;
+  const gchar *name;
+  const gchar *alt_name;
   gint type;
   gint id;
 } ValuePairSpec;
@@ -227,7 +227,7 @@ vp_results_insert(VPResults *results, GString *name, TypeHint type_hint, GString
 }
 
 static GString *
-vp_transform_apply (ValuePairs *vp, gchar *key)
+vp_transform_apply (ValuePairs *vp, const gchar *key)
 {
   gint i;
   GString *result = scratch_buffers_alloc();
@@ -302,7 +302,7 @@ vp_msg_nvpairs_foreach(NVHandle handle, gchar *name,
 }
 
 static gboolean
-vp_find_in_set(ValuePairs *vp, gchar *name, gboolean exclude)
+vp_find_in_set(ValuePairs *vp, const gchar *name, gboolean exclude)
 {
   guint j;
   gboolean included = exclude;
@@ -822,8 +822,17 @@ value_pairs_add_scope(ValuePairs *vp, const gchar *scope)
 {
   gboolean result;
 
-  result = cfg_process_flag(value_pair_scope, vp, scope);
-  vp_update_builtin_list_of_values(vp);
+  if (strcmp(scope, "none") != 0)
+    {
+      result = cfg_process_flag(value_pair_scope, vp, scope);
+      vp_update_builtin_list_of_values(vp);
+    }
+  else
+    {
+      result = TRUE;
+      vp->scopes = 0;
+      vp_update_builtin_list_of_values(vp);
+    }
   return result;
 }
 
@@ -948,7 +957,7 @@ value_pairs_init_set(ValuePairSpec *set)
   for (i = 0; set[i].name; i++)
     {
       guint id;
-      gchar *name;
+      const gchar *name;
 
       name = set[i].alt_name ? set[i].alt_name : set[i].name;
 
