@@ -29,6 +29,7 @@
 #include "contextual-data-record-scanner.h"
 #include "add-contextual-data-selector.h"
 #include "add-contextual-data-template-selector.h"
+#include "add-contextual-data-filter-selector.h"
 #include "template/templates.h"
 #include "context-info-db.h"
 #include "pathutils.h"
@@ -44,6 +45,7 @@ typedef struct AddContextualData
   gchar *default_selector;
   gchar *filename;
   gchar *prefix;
+  gboolean ignore_case;
 } AddContextualData;
 
 void
@@ -89,6 +91,21 @@ add_contextual_data_set_selector(LogParser *p, AddContextualDataSelector *select
   self->selector = selector;
 }
 
+void
+add_contextual_data_set_selector_filter(LogParser *p, const gchar *filename)
+{
+  AddContextualData *self = (AddContextualData *) p;
+  add_contextual_data_selector_free(self->selector);
+  self->selector = add_contextual_data_selector_filter_new(log_pipe_get_config(&p->super), filename);
+}
+
+void
+add_contextual_data_set_ignore_case(LogParser *p, gboolean ignore)
+{
+  AddContextualData *self = (AddContextualData *)p;
+  self->ignore_case = ignore;
+}
+
 static gboolean
 _is_default_selector_set(const AddContextualData *self)
 {
@@ -99,7 +116,7 @@ static void
 _add_context_data_to_message(gpointer pmsg, const ContextualDataRecord *record)
 {
   LogMessage *msg = (LogMessage *) pmsg;
-  log_msg_set_value_by_name(msg, record->name->str, record->value->str, record->value->len);
+  log_msg_set_value_by_name(msg, record->name->str, record->value->str, (gssize)record->value->len);
 }
 
 static gboolean
@@ -115,6 +132,12 @@ _process(LogParser *s, LogMessage **pmsg,
   if (!context_info_db_contains(self->context_info_db, selector) && _is_default_selector_set(self))
     selector = self->default_selector;
 
+  msg_trace("add-contextual-data(): message processing",
+            evt_tag_str("input", input),
+            evt_tag_str("resolved_selector", resolved_selector),
+            evt_tag_str("selector", selector),
+            evt_tag_printf("msg", "%p", *pmsg));
+
   if (selector)
     context_info_db_foreach_record(self->context_info_db, selector,
                                    _add_context_data_to_message,
@@ -146,6 +169,7 @@ _clone(LogPipe *s)
   add_contextual_data_set_filename(&cloned->super, self->filename);
   add_contextual_data_set_database_default_selector(&cloned->super,
                                                     self->default_selector);
+  add_contextual_data_set_ignore_case(&cloned->super, self->ignore_case);
   cloned->selector = add_contextual_data_selector_clone(self->selector, s->cfg);
 
   return &cloned->super.super;
@@ -203,11 +227,11 @@ _get_scanner(AddContextualData *self)
 {
   const gchar *type = get_filename_extension(self->filename);
   ContextualDataRecordScanner *scanner =
-    create_contextual_data_record_scanner_by_type(type);
+    create_contextual_data_record_scanner_by_type(self->filename, type);
 
   if (!scanner)
     {
-      msg_error("Unknown file extension",
+      msg_error("add-contextual-data(): unknown file extension, only files with a .csv extension are supported",
                 evt_tag_str("filename", self->filename));
       return NULL;
     }
@@ -228,8 +252,9 @@ _load_context_info_db(AddContextualData *self)
   FILE *f = _open_data_file(self->filename);
   if (!f)
     {
-      msg_error("Error loading add_contextual_data database",
-                evt_tag_str("filename", self->filename));
+      msg_error("add-contextual-data(): Error opening database",
+                evt_tag_str("filename", self->filename),
+                evt_tag_error("error"));
       contextual_data_record_scanner_free(scanner);
       return FALSE;
     }
@@ -241,7 +266,8 @@ _load_context_info_db(AddContextualData *self)
   fclose(f);
   if (!tag_db_loaded)
     {
-      msg_error("Error while parsing add_contextual_data database");
+      msg_error("add-contextual-data(): Error while parsing database",
+                evt_tag_str("filename", self->filename));
       return FALSE;
     }
 
@@ -254,15 +280,18 @@ _init_context_info_db(AddContextualData *self)
   if (self->selector && add_contextual_data_selector_is_ordering_required(self->selector))
     context_info_db_enable_ordering(self->context_info_db);
 
+  context_info_db_set_ignore_case(self->context_info_db, self->ignore_case);
+  if (!context_info_db_is_loaded(self->context_info_db))
+    context_info_db_init(self->context_info_db);
+
   if (self->filename == NULL)
     {
-      msg_error("No database file set.");
+      msg_error("add-contextual-data(): No database file set, specifying the database() option is mandatory");
       return FALSE;
     }
 
   if (!context_info_db_is_loaded(self->context_info_db) && !_load_context_info_db(self))
     {
-      msg_error("Failed to load the database file.");
       return FALSE;
     }
 
