@@ -50,6 +50,12 @@
 #  define _GNU_SOURCE 1
 #endif
 
+typedef struct _AFInetDestDriverTLSVerifyData
+{
+  TLSContext *tls_context;
+  gchar *hostname;
+} AFInetDestDriverTLSVerifyData;
+
 void
 afinet_dd_set_localip(LogDriver *s, gchar *ip)
 {
@@ -95,14 +101,13 @@ afinet_dd_set_spoof_source(LogDriver *s, gboolean enable)
 static gint
 afinet_dd_verify_callback(gint ok, X509_STORE_CTX *ctx, gpointer user_data)
 {
-  AFInetDestDriver *self G_GNUC_UNUSED = (AFInetDestDriver *) user_data;
-  TransportMapperInet *transport_mapper_inet = (TransportMapperInet *) self->super.transport_mapper;
+  AFInetDestDriverTLSVerifyData *self G_GNUC_UNUSED = (AFInetDestDriverTLSVerifyData *) user_data;
 
   X509 *current_cert = X509_STORE_CTX_get_current_cert(ctx);
   X509 *cert = X509_STORE_CTX_get0_cert(ctx);
 
   if (ok && current_cert == cert && self->hostname
-      && (tls_context_get_verify_mode(transport_mapper_inet->tls_context) & TVM_TRUSTED))
+      && (tls_context_get_verify_mode(self->tls_context) & TVM_TRUSTED))
     {
       ok = tls_verify_certificate_name(cert, self->hostname);
     }
@@ -110,22 +115,100 @@ afinet_dd_verify_callback(gint ok, X509_STORE_CTX *ctx, gpointer user_data)
   return ok;
 }
 
+static AFInetDestDriverTLSVerifyData *
+afinet_dd_tls_verify_data_new(TLSContext *ctx, const gchar *hostname)
+{
+  AFInetDestDriverTLSVerifyData *self = g_new0(AFInetDestDriverTLSVerifyData, 1);
+
+  self->tls_context = tls_context_ref(ctx);
+  self->hostname = g_strdup(hostname);
+  return self;
+}
+
+static void
+afinet_dd_tls_verify_data_free(gpointer s)
+{
+  AFInetDestDriverTLSVerifyData *self = (AFInetDestDriverTLSVerifyData *)s;
+
+  g_assert(self);
+
+  if (self)
+    {
+      tls_context_unref(self->tls_context);
+      g_free(self->hostname);
+      g_free(self);
+    }
+}
+
+static gboolean
+_is_failover_used(const AFInetDestDriver *self)
+{
+  return self->failover != NULL;
+}
+
+static const gchar *
+_afinet_dd_get_hostname(const AFInetDestDriver *self)
+{
+  if (!_is_failover_used(self))
+    return self->primary;
+
+  return afinet_dd_failover_get_hostname(self->failover);
+}
+
 void
 afinet_dd_set_tls_context(LogDriver *s, TLSContext *tls_context)
 {
   AFInetDestDriver *self = (AFInetDestDriver *) s;
-  transport_mapper_inet_set_tls_context((TransportMapperInet *) self->super.transport_mapper, tls_context,
-                                        afinet_dd_verify_callback, self);
+  AFInetDestDriverTLSVerifyData *verify_data;
+  TLSVerifier *verifier;
+
+  verify_data = afinet_dd_tls_verify_data_new(tls_context, _afinet_dd_get_hostname(self));
+  verifier = tls_verifier_new(afinet_dd_verify_callback, verify_data, afinet_dd_tls_verify_data_free);
+  transport_mapper_inet_set_tls_context((TransportMapperInet *) self->super.transport_mapper, tls_context, verifier);
 }
 
-static LogWriter *
-afinet_dd_construct_writer(AFSocketDestDriver *s)
+void
+afinet_dd_enable_failover(LogDriver *s)
 {
-  AFInetDestDriver *self G_GNUC_UNUSED = (AFInetDestDriver *) s;
-  LogWriter *writer;
-  TransportMapperInet *transport_mapper_inet G_GNUC_UNUSED = ((TransportMapperInet *) (self->super.transport_mapper));
+  AFInetDestDriver *self = (AFInetDestDriver *) s;
+  if (self->failover)
+    return;
+  self->failover = afinet_dd_failover_new(self->primary);
+}
 
-  writer = afsocket_dd_construct_writer_method(s);
+void
+afinet_dd_add_failovers(LogDriver *s, GList *failovers)
+{
+  AFInetDestDriver *self = (AFInetDestDriver *)s;
+  g_assert(self->failover != NULL);
+  afinet_dd_failover_add_servers(self->failover, failovers);
+}
+
+void
+afinet_dd_enable_failback(LogDriver *s)
+{
+  AFInetDestDriver *self = (AFInetDestDriver *) s;
+  g_assert(self->failover != NULL);
+  afinet_dd_failover_enable_failback(self->failover, &self->super, afsocket_dd_connected_with_fd);
+}
+
+void
+afinet_dd_set_failback_tcp_probe_interval(LogDriver *s, gint tcp_probe_interval)
+{
+  AFInetDestDriver *self = (AFInetDestDriver *) s;
+  afinet_dd_failover_set_tcp_probe_interval(self->failover, tcp_probe_interval);
+}
+
+void
+afinet_dd_set_failback_successful_probes_required(LogDriver *s, gint successful_probes_required)
+{
+  AFInetDestDriver *self = (AFInetDestDriver *) s;
+  afinet_dd_failover_set_successful_probes_required(self->failover, successful_probes_required);
+}
+
+static void
+_disable_connection_closure_on_input(LogWriter *writer)
+{
   /* SSL is duplex, so we can certainly expect input from the server, which
    * would cause the LogWriter to close this connection.  In a better world
    * LW_DETECT_EOF would be implemented by the LogProto class and would
@@ -133,43 +216,48 @@ afinet_dd_construct_writer(AFSocketDestDriver *s)
    * (and possibly for all eternity :)
    */
 
+  log_writer_set_flags(writer, log_writer_get_flags(writer) & ~LW_DETECT_EOF);
+}
+
+static LogWriter *
+afinet_dd_construct_writer(AFSocketDestDriver *s)
+{
+  AFInetDestDriver *self = (AFInetDestDriver *) s;
+  TransportMapperInet *transport_mapper_inet = ((TransportMapperInet *) (self->super.transport_mapper));
+
+  LogWriter *writer = afsocket_dd_construct_writer_method(s);
+
   if (((self->super.transport_mapper->sock_type == SOCK_STREAM) && transport_mapper_inet->tls_context))
-    log_writer_set_flags(writer, log_writer_get_flags(writer) & ~LW_DETECT_EOF);
+    _disable_connection_closure_on_input(writer);
 
   return writer;
 }
 
-static const gint
+static gint
 _determine_port(const AFInetDestDriver *self)
 {
-  gint port = 0;
-
-  if (!self->dest_port)
-    port = transport_mapper_inet_get_server_port(self->super.transport_mapper);
-  else
-    port = afinet_lookup_service(self->super.transport_mapper, self->dest_port);
-
-  return port;
+  return afinet_determine_port(self->super.transport_mapper, self->dest_port);
 }
 
 static gboolean
-afinet_dd_setup_addresses(AFSocketDestDriver *s)
+_setup_bind_addr(AFInetDestDriver *self)
 {
-  AFInetDestDriver *self = (AFInetDestDriver *) s;
-
-  if (!afsocket_dd_setup_addresses_method(s))
-    return FALSE;
-
   g_sockaddr_unref(self->super.bind_addr);
-  g_sockaddr_unref(self->super.dest_addr);
-
   if (!resolve_hostname_to_sockaddr(&self->super.bind_addr, self->super.transport_mapper->address_family, self->bind_ip))
     return FALSE;
 
   if (self->bind_port)
     g_sockaddr_set_port(self->super.bind_addr, afinet_lookup_service(self->super.transport_mapper, self->bind_port));
 
-  if (!resolve_hostname_to_sockaddr(&self->super.dest_addr, self->super.transport_mapper->address_family, self->hostname))
+  return TRUE;
+}
+
+static gboolean
+_setup_dest_addr(AFInetDestDriver *self)
+{
+  g_sockaddr_unref(self->super.dest_addr);
+  if (!resolve_hostname_to_sockaddr(&self->super.dest_addr, self->super.transport_mapper->address_family,
+                                    _afinet_dd_get_hostname(self)))
     return FALSE;
 
   if (!self->dest_port)
@@ -188,19 +276,65 @@ afinet_dd_setup_addresses(AFSocketDestDriver *s)
   return TRUE;
 }
 
+static gboolean
+afinet_dd_setup_addresses(AFSocketDestDriver *s)
+{
+  AFInetDestDriver *self = (AFInetDestDriver *) s;
+
+  if (!afsocket_dd_setup_addresses_method(s))
+    return FALSE;
+
+  if (self->super.proto_factory->default_inet_port)
+    transport_mapper_inet_set_server_port(self->super.transport_mapper, self->super.proto_factory->default_inet_port);
+
+  if (!_setup_bind_addr(self))
+    return FALSE;
+
+  if (_is_failover_used(self))
+    afinet_dd_failover_next(self->failover);
+
+  if (!_setup_dest_addr(self))
+    return FALSE;
+
+  return TRUE;
+}
+
 static const gchar *
 afinet_dd_get_dest_name(const AFSocketDestDriver *s)
 {
   const AFInetDestDriver *self = (const AFInetDestDriver *)s;
   static gchar buf[256];
+  const gchar *hostname = _afinet_dd_get_hostname(self);
 
-  if (strchr(self->hostname, ':') != NULL)
-    g_snprintf(buf, sizeof(buf), "[%s]:%d", self->hostname, _determine_port(self));
+  if (strchr(hostname, ':') != NULL)
+    g_snprintf(buf, sizeof(buf), "[%s]:%d", hostname, _determine_port(self));
   else
-    g_snprintf(buf, sizeof(buf), "%s:%d", self->hostname, _determine_port(self));
+    g_snprintf(buf, sizeof(buf), "%s:%d", hostname, _determine_port(self));
   return buf;
 }
 
+static inline void
+_libnet_destroy_when_spoof_source_enabled(AFInetDestDriver *self)
+{
+#if SYSLOG_NG_ENABLE_SPOOF_SOURCE
+  if (self->lnet_ctx)
+    libnet_destroy(self->lnet_ctx);
+#endif
+}
+
+static gboolean
+afinet_dd_deinit(LogPipe *s)
+{
+  AFInetDestDriver *self = (AFInetDestDriver *) s;
+
+  if (_is_failover_used(self))
+    afinet_dd_failover_deinit(self->failover);
+
+  _libnet_destroy_when_spoof_source_enabled(self);
+
+  return afsocket_dd_deinit(s);
+}
+
 static gboolean
 afinet_dd_init(LogPipe *s)
 {
@@ -208,7 +342,7 @@ afinet_dd_init(LogPipe *s)
 
 #if SYSLOG_NG_ENABLE_SPOOF_SOURCE
   if (self->spoof_source)
-    self->super.connections_kept_alive_accross_reloads = TRUE;
+    self->super.connections_kept_alive_across_reloads = TRUE;
 #endif
 
   if (!afsocket_dd_init(s))
@@ -223,7 +357,7 @@ afinet_dd_init(LogPipe *s)
           cap_t saved_caps;
 
           saved_caps = g_process_cap_save();
-          g_process_cap_modify(CAP_NET_RAW, TRUE);
+          g_process_enable_cap("cap_net_raw");
           self->lnet_ctx = libnet_init(self->super.bind_addr->sa.sa_family == AF_INET ? LIBNET_RAW4 : LIBNET_RAW6, NULL, error);
           g_process_cap_restore(saved_caps);
           if (!self->lnet_ctx)
@@ -235,6 +369,20 @@ afinet_dd_init(LogPipe *s)
     }
 #endif
 
+  if (_is_failover_used(self))
+    {
+      FailoverTransportMapper ftm =
+      {
+        .transport_mapper = self->super.transport_mapper,
+        .socket_options = self->super.socket_options,
+        .dest_port = self->dest_port,
+        .bind_ip = self->bind_ip,
+        .bind_port = self->bind_port
+      };
+
+      afinet_dd_failover_init(self->failover, s->expr_node, &ftm);
+    }
+
   return TRUE;
 }
 
@@ -350,68 +498,91 @@ afinet_dd_construct_ipv6_packet(AFInetDestDriver *self, LogMessage *msg, GString
 }
 #endif
 
+static inline gboolean
+afinet_dd_construct_ip_packet(AFInetDestDriver *self, LogMessage *msg, GString *msg_line)
+{
+  switch (self->super.dest_addr->sa.sa_family)
+    {
+    case AF_INET:
+      return afinet_dd_construct_ipv4_packet(self, msg, msg_line);
+#if SYSLOG_NG_ENABLE_IPV6
+    case AF_INET6:
+      return afinet_dd_construct_ipv6_packet(self, msg, msg_line);
+#endif
+    default:
+      g_assert_not_reached();
+    }
+
+  return FALSE;
+}
+
+static gboolean
+afinet_dd_spoof_write_message(AFInetDestDriver *self, LogMessage *msg, const LogPathOptions *path_options)
+{
+  g_assert(self->super.transport_mapper->sock_type == SOCK_DGRAM);
+
+  g_static_mutex_lock(&self->lnet_lock);
+
+  if (!self->lnet_buffer)
+    self->lnet_buffer = g_string_sized_new(self->spoof_source_maxmsglen);
+
+  log_writer_format_log(self->super.writer, msg, self->lnet_buffer);
+
+  if (self->lnet_buffer->len > self->spoof_source_maxmsglen)
+    g_string_truncate(self->lnet_buffer, self->spoof_source_maxmsglen);
+
+  gboolean success = afinet_dd_construct_ip_packet(self, msg, self->lnet_buffer);
+
+  if (!success)
+    goto finish;
+
+  success = libnet_write(self->lnet_ctx) >= 0;
+
+  if (!success)
+    {
+      msg_error("Error sending raw frame", evt_tag_str("error", libnet_geterror(self->lnet_ctx)));
+      goto finish;
+    }
+
+  /* we have finished processing msg */
+  log_msg_ack(msg, path_options, AT_PROCESSED);
+  log_msg_unref(msg);
+
+finish:
+  g_static_mutex_unlock(&self->lnet_lock);
+  return success;
+}
+
+static inline gboolean
+_is_message_spoofable(LogMessage *msg)
+{
+  return msg->saddr && (msg->saddr->sa.sa_family == AF_INET || msg->saddr->sa.sa_family == AF_INET6);
+}
+
+static inline gboolean
+_is_spoof_source_enabled(AFInetDestDriver *self)
+{
+  return self->spoof_source && self->lnet_ctx;
+}
+
 #endif
 
 static void
-afinet_dd_queue(LogPipe *s, LogMessage *msg, const LogPathOptions *path_options, gpointer user_data)
+afinet_dd_queue(LogPipe *s, LogMessage *msg, const LogPathOptions *path_options)
 {
 #if SYSLOG_NG_ENABLE_SPOOF_SOURCE
   AFInetDestDriver *self = (AFInetDestDriver *) s;
 
-  /* NOTE: this code should probably become a LogTransport instance so that
+  /* NOTE: This code should probably be moved to the LogProto layer so that
    * spoofed packets are also going through the LogWriter queue */
 
-  if (self->spoof_source && self->lnet_ctx && msg->saddr && (msg->saddr->sa.sa_family == AF_INET
-                                                             || msg->saddr->sa.sa_family == AF_INET6) && log_writer_opened(self->super.writer))
+  if (_is_spoof_source_enabled(self) && _is_message_spoofable(msg) && log_writer_opened(self->super.writer))
     {
-      gboolean success = FALSE;
-
-      g_assert(self->super.transport_mapper->sock_type == SOCK_DGRAM);
-
-      g_static_mutex_lock(&self->lnet_lock);
-
-      if (!self->lnet_buffer)
-        self->lnet_buffer = g_string_sized_new(self->spoof_source_maxmsglen);
-
-      log_writer_format_log(self->super.writer, msg, self->lnet_buffer);
-
-      if (self->lnet_buffer->len > self->spoof_source_maxmsglen)
-        g_string_truncate(self->lnet_buffer, self->spoof_source_maxmsglen);
-
-      switch (self->super.dest_addr->sa.sa_family)
-        {
-        case AF_INET:
-          success = afinet_dd_construct_ipv4_packet(self, msg, self->lnet_buffer);
-          break;
-#if SYSLOG_NG_ENABLE_IPV6
-        case AF_INET6:
-          success = afinet_dd_construct_ipv6_packet(self, msg, self->lnet_buffer);
-          break;
-#endif
-        default:
-          g_assert_not_reached();
-        }
-      if (success)
-        {
-          if (libnet_write(self->lnet_ctx) >= 0)
-            {
-              /* we have finished processing msg */
-              log_msg_ack(msg, path_options, AT_PROCESSED);
-              log_msg_unref(msg);
-
-              g_static_mutex_unlock(&self->lnet_lock);
-              return;
-            }
-          else
-            {
-              msg_error("Error sending raw frame",
-                        evt_tag_str("error", libnet_geterror(self->lnet_ctx)));
-            }
-        }
-      g_static_mutex_unlock(&self->lnet_lock);
+      if (afinet_dd_spoof_write_message(self, msg, path_options))
+        return;
     }
 #endif
-  log_dest_driver_queue_method(s, msg, path_options, user_data);
+  log_dest_driver_queue_method(s, msg, path_options);
 }
 
 void
@@ -419,7 +590,10 @@ afinet_dd_free(LogPipe *s)
 {
   AFInetDestDriver *self = (AFInetDestDriver *) s;
 
-  g_free(self->hostname);
+  g_free(self->primary);
+  afinet_dd_failover_free(self->failover);
+
+
   g_free(self->bind_ip);
   g_free(self->bind_port);
   g_free(self->dest_port);
@@ -431,7 +605,6 @@ afinet_dd_free(LogPipe *s)
   afsocket_dd_free(s);
 }
 
-
 static AFInetDestDriver *
 afinet_dd_new_instance(TransportMapper *transport_mapper, gchar *hostname, GlobalConfig *cfg)
 {
@@ -439,13 +612,14 @@ afinet_dd_new_instance(TransportMapper *transport_mapper, gchar *hostname, Globa
 
   afsocket_dd_init_instance(&self->super, socket_options_inet_new(), transport_mapper, cfg);
   self->super.super.super.super.init = afinet_dd_init;
+  self->super.super.super.super.deinit = afinet_dd_deinit;
   self->super.super.super.super.queue = afinet_dd_queue;
   self->super.super.super.super.free_fn = afinet_dd_free;
   self->super.construct_writer = afinet_dd_construct_writer;
   self->super.setup_addresses = afinet_dd_setup_addresses;
   self->super.get_dest_name = afinet_dd_get_dest_name;
 
-  self->hostname = g_strdup(hostname);
+  self->primary = g_strdup(hostname);
 
 #if SYSLOG_NG_ENABLE_SPOOF_SOURCE
   g_static_mutex_init(&self->lnet_lock);
@@ -481,10 +655,14 @@ afinet_dd_new_udp6(gchar *host, GlobalConfig *cfg)
 static LogWriter *
 afinet_dd_syslog_construct_writer(AFSocketDestDriver *s)
 {
-  AFInetDestDriver *self G_GNUC_UNUSED = (AFInetDestDriver *) s;
-  LogWriter *writer;
+  AFInetDestDriver *self = (AFInetDestDriver *) s;
+  TransportMapperInet *transport_mapper_inet = ((TransportMapperInet *) (self->super.transport_mapper));
+
+  LogWriter *writer = afsocket_dd_construct_writer_method(s);
+
+  if (((self->super.transport_mapper->sock_type == SOCK_STREAM) && transport_mapper_inet->tls_context))
+    _disable_connection_closure_on_input(writer);
 
-  writer = afsocket_dd_construct_writer_method(s);
   log_writer_set_flags(writer, log_writer_get_flags(writer) | LW_SYSLOG_PROTOCOL);
   return writer;
 }
