@@ -79,7 +79,7 @@ afsocket_dd_set_keep_alive(LogDriver *s, gboolean enable)
 {
   AFSocketDestDriver *self = (AFSocketDestDriver *) s;
 
-  self->connections_kept_alive_accross_reloads = enable;
+  self->connections_kept_alive_across_reloads = enable;
 }
 
 static const gchar *_module_name = "afsocket_dd";
@@ -137,8 +137,7 @@ afsocket_dd_stats_instance(AFSocketDestDriver *self)
   return buf;
 }
 
-static gboolean afsocket_dd_connected(AFSocketDestDriver *self);
-static void afsocket_dd_reconnect(AFSocketDestDriver *self);
+static void _afsocket_dd_connection_in_progress(AFSocketDestDriver *self);
 static void afsocket_dd_try_connect(AFSocketDestDriver *self);
 static gboolean afsocket_dd_setup_connection(AFSocketDestDriver *self);
 
@@ -147,7 +146,7 @@ afsocket_dd_init_watches(AFSocketDestDriver *self)
 {
   IV_FD_INIT(&self->connect_fd);
   self->connect_fd.cookie = self;
-  self->connect_fd.handler_out = (void (*)(void *)) afsocket_dd_connected;
+  self->connect_fd.handler_out = (void (*)(void *)) _afsocket_dd_connection_in_progress;
 
   IV_TIMER_INIT(&self->reconnect_timer);
   self->reconnect_timer.cookie = self;
@@ -166,7 +165,7 @@ afsocket_dd_start_watches(AFSocketDestDriver *self)
   iv_fd_register(&self->connect_fd);
 }
 
-static void
+void
 afsocket_dd_stop_watches(AFSocketDestDriver *self)
 {
   main_loop_assert_main_thread();
@@ -179,6 +178,7 @@ afsocket_dd_stop_watches(AFSocketDestDriver *self)
       msg_verbose("Closing connecting fd",
                   evt_tag_int("fd", self->fd));
       close(self->fd);
+      self->fd = -1;
     }
   if (iv_timer_registered(&self->reconnect_timer))
     iv_timer_unregister(&self->reconnect_timer);
@@ -194,7 +194,7 @@ afsocket_dd_start_reconnect_timer(AFSocketDestDriver *self)
   iv_validate_now();
 
   self->reconnect_timer.expires = iv_now;
-  timespec_add_msec(&self->reconnect_timer.expires, self->time_reopen * 1000);
+  timespec_add_msec(&self->reconnect_timer.expires, self->time_reopen * 1000L);
   iv_timer_register(&self->reconnect_timer);
 }
 
@@ -207,14 +207,52 @@ afsocket_dd_construct_transport(AFSocketDestDriver *self, gint fd)
 static gboolean
 afsocket_dd_connected(AFSocketDestDriver *self)
 {
-  gchar buf1[256], buf2[256];
-  int error = 0;
-  socklen_t errorlen = sizeof(error);
+  GlobalConfig *cfg = log_pipe_get_config(&self->super.super.super);
   LogTransport *transport;
   LogProtoClient *proto;
+  gchar buf1[256], buf2[256];
 
   main_loop_assert_main_thread();
 
+  msg_notice("Syslog connection established",
+             evt_tag_int("fd", self->fd),
+             evt_tag_str("server", g_sockaddr_format(self->dest_addr, buf2, sizeof(buf2), GSA_FULL)),
+             evt_tag_str("local", g_sockaddr_format(self->bind_addr, buf1, sizeof(buf1), GSA_FULL)));
+
+  transport = afsocket_dd_construct_transport(self, self->fd);
+  if (!transport)
+    return FALSE;
+
+  proto = log_proto_client_factory_construct(self->proto_factory, transport, &self->writer_options.proto_options.super);
+
+  log_proto_client_restart_with_state(proto, cfg->state, afsocket_dd_format_connections_name(self));
+  log_writer_reopen(self->writer, proto);
+  return TRUE;
+}
+
+void
+afsocket_dd_connected_with_fd(gpointer s, gint fd, GSockAddr *saddr)
+{
+  AFSocketDestDriver *self = (AFSocketDestDriver *)s;
+  afsocket_dd_stop_watches(self);
+  g_sockaddr_unref(self->dest_addr);
+  self->dest_addr = saddr;
+  self->fd = fd;
+  if (!afsocket_dd_connected(self))
+    {
+      close(self->fd);
+      self->fd = -1;
+      afsocket_dd_start_reconnect_timer(self);
+    }
+}
+
+static void
+_afsocket_dd_connection_in_progress(AFSocketDestDriver *self)
+{
+  gchar buf[256];
+  int error = 0;
+  socklen_t errorlen = sizeof(error);
+
   if (iv_fd_registered(&self->connect_fd))
     iv_fd_unregister(&self->connect_fd);
 
@@ -224,8 +262,8 @@ afsocket_dd_connected(AFSocketDestDriver *self)
         {
           msg_error("getsockopt(SOL_SOCKET, SO_ERROR) failed for connecting socket",
                     evt_tag_int("fd", self->fd),
-                    evt_tag_str("server", g_sockaddr_format(self->dest_addr, buf2, sizeof(buf2), GSA_FULL)),
-                    evt_tag_errno(EVT_TAG_OSERROR, errno),
+                    evt_tag_str("server", g_sockaddr_format(self->dest_addr, buf, sizeof(buf), GSA_FULL)),
+                    evt_tag_error(EVT_TAG_OSERROR),
                     evt_tag_int("time_reopen", self->time_reopen));
           goto error_reconnect;
         }
@@ -233,30 +271,20 @@ afsocket_dd_connected(AFSocketDestDriver *self)
         {
           msg_error("Syslog connection failed",
                     evt_tag_int("fd", self->fd),
-                    evt_tag_str("server", g_sockaddr_format(self->dest_addr, buf2, sizeof(buf2), GSA_FULL)),
+                    evt_tag_str("server", g_sockaddr_format(self->dest_addr, buf, sizeof(buf), GSA_FULL)),
                     evt_tag_errno(EVT_TAG_OSERROR, error),
                     evt_tag_int("time_reopen", self->time_reopen));
           goto error_reconnect;
         }
     }
-  msg_notice("Syslog connection established",
-             evt_tag_int("fd", self->fd),
-             evt_tag_str("server", g_sockaddr_format(self->dest_addr, buf2, sizeof(buf2), GSA_FULL)),
-             evt_tag_str("local", g_sockaddr_format(self->bind_addr, buf1, sizeof(buf1), GSA_FULL)));
 
-  transport = afsocket_dd_construct_transport(self, self->fd);
-  if (!transport)
-    goto error_reconnect;
+  if (afsocket_dd_connected(self))
+    return;
 
-  proto = log_proto_client_factory_construct(self->proto_factory, transport, &self->writer_options.proto_options.super);
-
-  log_writer_reopen(self->writer, proto);
-  return TRUE;
 error_reconnect:
   close(self->fd);
   self->fd = -1;
   afsocket_dd_start_reconnect_timer(self);
-  return FALSE;
 }
 
 static gboolean
@@ -282,7 +310,12 @@ afsocket_dd_start_connect(AFSocketDestDriver *self)
   if (rc == G_IO_STATUS_NORMAL)
     {
       self->fd = sock;
-      afsocket_dd_connected(self);
+      if (!afsocket_dd_connected(self))
+        {
+          close(self->fd);
+          self->fd = -1;
+          return FALSE;
+        }
     }
   else if (rc == G_IO_STATUS_ERROR && errno == EINPROGRESS)
     {
@@ -298,7 +331,7 @@ afsocket_dd_start_connect(AFSocketDestDriver *self)
                 evt_tag_int("fd", sock),
                 evt_tag_str("server", g_sockaddr_format(self->dest_addr, buf2, sizeof(buf2), GSA_FULL)),
                 evt_tag_str("local", g_sockaddr_format(self->bind_addr, buf1, sizeof(buf1), GSA_FULL)),
-                evt_tag_errno(EVT_TAG_OSERROR, errno));
+                evt_tag_error(EVT_TAG_OSERROR));
       close(sock);
       return FALSE;
     }
@@ -307,10 +340,9 @@ afsocket_dd_start_connect(AFSocketDestDriver *self)
 }
 
 static void
-afsocket_dd_reconnect(AFSocketDestDriver *self)
+_dd_reconnect(AFSocketDestDriver *self, gboolean request_setup_addr)
 {
-  if (!afsocket_dd_setup_addresses(self) ||
-      !afsocket_dd_start_connect(self))
+  if ((request_setup_addr && !afsocket_dd_setup_addresses(self)) || !afsocket_dd_start_connect(self))
     {
       msg_error("Initiating connection failed, reconnecting",
                 evt_tag_int("time_reopen", self->time_reopen));
@@ -318,11 +350,28 @@ afsocket_dd_reconnect(AFSocketDestDriver *self)
     }
 }
 
+static void
+_dd_reconnect_with_setup_addresses(AFSocketDestDriver *self)
+{
+  _dd_reconnect(self, TRUE);
+}
+
+static void
+_dd_reconnect_with_current_addresses(AFSocketDestDriver *self)
+{
+  _dd_reconnect(self, FALSE);
+}
+
+void
+afsocket_dd_reconnect(AFSocketDestDriver *self)
+{
+  _dd_reconnect_with_setup_addresses(self);
+}
+
 static void
 afsocket_dd_try_connect(AFSocketDestDriver *self)
 {
-  if (!afsocket_dd_setup_addresses(self) ||
-      !afsocket_dd_setup_connection(self))
+  if ((!afsocket_dd_setup_addresses(self)) || !afsocket_dd_setup_connection(self))
     {
       msg_error("Initiating connection failed, reconnecting",
                 evt_tag_int("time_reopen", self->time_reopen));
@@ -337,13 +386,18 @@ afsocket_dd_setup_proto_factory(AFSocketDestDriver *self)
 {
   GlobalConfig *cfg = log_pipe_get_config(&self->super.super.super);
 
-  self->proto_factory = log_proto_client_get_factory(&cfg->plugin_context, self->transport_mapper->logproto);
+  if (!self->proto_factory)
+    self->proto_factory = log_proto_client_get_factory(&cfg->plugin_context, self->transport_mapper->logproto);
+
   if (!self->proto_factory)
     {
       msg_error("Unknown value specified in the transport() option, no such LogProto plugin found",
                 evt_tag_str("transport", self->transport_mapper->logproto));
       return FALSE;
     }
+
+  self->transport_mapper->create_multitransport = self->proto_factory->use_multitransport;
+
   return TRUE;
 }
 
@@ -450,12 +504,68 @@ afsocket_dd_setup_connection(AFSocketDestDriver *self)
   self->time_reopen = cfg->time_reopen;
 
   if (!log_writer_opened(self->writer))
-    afsocket_dd_reconnect(self);
+    _dd_reconnect_with_current_addresses(self);
 
   self->connection_initialized = TRUE;
   return TRUE;
 }
 
+static gboolean
+_finalize_init(gpointer arg)
+{
+  AFSocketDestDriver *self = (AFSocketDestDriver *)arg;
+  afsocket_dd_try_connect(self);
+  return TRUE;
+}
+
+static gboolean
+_dd_init_stream(AFSocketDestDriver *self)
+{
+  if (!afsocket_dd_setup_writer(self))
+    return FALSE;
+
+  return transport_mapper_async_init(self->transport_mapper, _finalize_init, self);
+}
+
+static gboolean
+_dd_init_dgram(AFSocketDestDriver *self)
+{
+  if (!transport_mapper_init(self->transport_mapper))
+    {
+      return FALSE;
+    }
+
+  if (!afsocket_dd_setup_writer(self))
+    {
+      return FALSE;
+    }
+
+  return _finalize_init(self);
+}
+
+static void
+_dd_rewind_stateless_proto_backlog(AFSocketDestDriver *self)
+{
+  if (!log_proto_client_factory_is_proto_stateful(self->proto_factory))
+    {
+      log_writer_msg_rewind(self->writer);
+    }
+}
+
+static gboolean
+_dd_init_socket(AFSocketDestDriver *self)
+{
+  switch (self->transport_mapper->sock_type)
+    {
+    case SOCK_STREAM:
+      return _dd_init_stream(self);
+
+    case SOCK_DGRAM:
+    default:
+      return _dd_init_dgram(self);
+    }
+}
+
 gboolean
 afsocket_dd_init(LogPipe *s)
 {
@@ -467,13 +577,13 @@ afsocket_dd_init(LogPipe *s)
       return FALSE;
     }
 
-  if (!transport_mapper_init(self->transport_mapper))
-    return FALSE;
+  if (!_dd_init_socket(self))
+    {
+      return FALSE;
+    }
 
-  if (!afsocket_dd_setup_writer(self))
-    return FALSE;
+  _dd_rewind_stateless_proto_backlog(self);
 
-  afsocket_dd_try_connect(self);
   return TRUE;
 }
 
@@ -489,7 +599,7 @@ afsocket_dd_save_connection(AFSocketDestDriver *self)
 {
   GlobalConfig *cfg = log_pipe_get_config(&self->super.super.super);
 
-  if (self->connections_kept_alive_accross_reloads)
+  if (self->connections_kept_alive_across_reloads)
     {
       ReloadStoreItem *item = _reload_store_item_new(self);
       cfg_persist_config_add(cfg, afsocket_dd_format_connections_name(self), item,
@@ -526,12 +636,14 @@ afsocket_dd_notify(LogPipe *s, gint notify_code, gpointer user_data)
     case NC_WRITE_ERROR:
       log_writer_reopen(self->writer, NULL);
 
-      msg_notice("Syslog connection broken",
+      msg_notice((notify_code == NC_CLOSE) ? "Syslog connection closed" : "Syslog connection broken",
                  evt_tag_int("fd", self->fd),
                  evt_tag_str("server", g_sockaddr_format(self->dest_addr, buf, sizeof(buf), GSA_FULL)),
                  evt_tag_int("time_reopen", self->time_reopen));
       afsocket_dd_start_reconnect_timer(self);
       break;
+    default:
+      break;
     }
 }
 
@@ -567,7 +679,7 @@ afsocket_dd_init_instance(AFSocketDestDriver *self,
   self->construct_writer = afsocket_dd_construct_writer_method;
   self->transport_mapper = transport_mapper;
   self->socket_options = socket_options;
-  self->connections_kept_alive_accross_reloads = TRUE;
+  self->connections_kept_alive_across_reloads = TRUE;
   self->time_reopen = cfg->time_reopen;
   self->connection_initialized = FALSE;
 
