@@ -57,9 +57,11 @@ static CfgLexerKeyword afsocket_keywords[] =
   { "ecdh_curve_list",    KW_ECDH_CURVE_LIST },
   { "curve_list",         KW_ECDH_CURVE_LIST, KWS_OBSOLETE, "ecdh_curve_list"},
   { "ssl_options",        KW_SSL_OPTIONS },
+  { "allow_compress",     KW_ALLOW_COMPRESS },
 
   { "localip",            KW_LOCALIP },
   { "ip",                 KW_IP },
+  { "interface",          KW_INTERFACE },
   { "localport",          KW_LOCALPORT },
   { "port",               KW_PORT },
   { "destport",           KW_DESTPORT },
@@ -70,6 +72,7 @@ static CfgLexerKeyword afsocket_keywords[] =
   { "so_rcvbuf",          KW_SO_RCVBUF },
   { "so_sndbuf",          KW_SO_SNDBUF },
   { "so_keepalive",       KW_SO_KEEPALIVE },
+  { "so_reuseport",       KW_SO_REUSEPORT },
   { "tcp_keep_alive",     KW_SO_KEEPALIVE }, /* old, once deprecated form, but revived in 3.4 */
   { "tcp_keepalive",      KW_SO_KEEPALIVE }, /* alias for so-keepalive, as tcp is the only option actually using it */
   { "tcp_keepalive_time", KW_TCP_KEEPALIVE_TIME },
@@ -82,6 +85,12 @@ static CfgLexerKeyword afsocket_keywords[] =
   { "listen_backlog",     KW_LISTEN_BACKLOG },
   { "keep_alive",         KW_KEEP_ALIVE },
   { "systemd_syslog",     KW_SYSTEMD_SYSLOG  },
+  { "failover_servers",   KW_FAILOVER_SERVERS, KWS_OBSOLETE, "failover-servers has been deprecated, try failover() and use servers() option inside it." },
+  { "failover",           KW_FAILOVER },
+  { "failback",           KW_FAILBACK },
+  { "servers",            KW_SERVERS },
+  { "tcp_probe_interval", KW_TCP_PROBE_INTERVAL },
+  { "successful_probes_required", KW_SUCCESSFUL_PROBES_REQUIRED },
   { NULL }
 };
 
