@@ -33,6 +33,7 @@
 #include "apphook.h"
 #include "timeutils.h"
 #include "mainloop-worker.h"
+#include "str-utils.h"
 
 #include <string.h>
 #include <errno.h>
@@ -42,6 +43,7 @@ static gboolean dbi_initialized = FALSE;
 static const char *s_oracle = "oracle";
 static const char *s_freetds = "freetds";
 static dbi_inst dbi_instance;
+static const gint DEFAULT_SQL_TX_SIZE = 100;
 
 #define MAX_FAILED_ATTEMPTS 3
 
@@ -174,26 +176,11 @@ afsql_dd_set_null_value(LogDriver *s, const gchar *null)
 }
 
 void
-afsql_dd_set_retries(LogDriver *s, gint num_retries)
-{
-  AFSqlDestDriver *self = (AFSqlDestDriver *) s;
-  self->num_retries = num_retries;
-}
-
-void
-afsql_dd_set_flush_lines(LogDriver *s, gint flush_lines)
+afsql_dd_set_ignore_tns_config(LogDriver *s, const gboolean ignore_tns_config)
 {
   AFSqlDestDriver *self = (AFSqlDestDriver *) s;
 
-  self->flush_lines = flush_lines;
-}
-
-void
-afsql_dd_set_flush_timeout(LogDriver *s, gint flush_timeout)
-{
-  AFSqlDestDriver *self = (AFSqlDestDriver *) s;
-
-  self->flush_timeout = flush_timeout;
+  self->ignore_tns_config = ignore_tns_config;
 }
 
 void
@@ -262,20 +249,6 @@ afsql_dd_run_query(AFSqlDestDriver *self, const gchar *query, gboolean silent, d
   return TRUE;
 }
 
-/**
- * afsql_dd_handle_transaction_error:
- *
- * Handle errors inside during a SQL transaction (e.g. INSERT or COMMIT failures).
- *
- * NOTE: This function can only be called from the database thread.
- **/
-static void
-afsql_dd_handle_transaction_error(AFSqlDestDriver *self)
-{
-  log_queue_rewind_backlog_all(self->queue);
-  self->flush_lines_queued = 0;
-}
-
 /**
  * afsql_dd_commit_transaction:
  *
@@ -294,14 +267,11 @@ afsql_dd_commit_transaction(AFSqlDestDriver *self)
   success = afsql_dd_run_query(self, "COMMIT", FALSE, NULL);
   if (success)
     {
-      log_queue_ack_backlog(self->queue, self->flush_lines_queued);
-      self->flush_lines_queued = 0;
       self->transaction_active = FALSE;
     }
   else
     {
       msg_error("SQL transaction commit failed, rewinding backlog and starting again");
-      afsql_dd_handle_transaction_error(self);
     }
   return success;
 }
@@ -653,30 +623,6 @@ afsql_dd_ensure_table_is_syslogng_conform(AFSqlDestDriver *self, GString *table)
   return success;
 }
 
-
-/**
- * afsql_dd_suspend:
- * timeout: in milliseconds
- *
- * This function is assumed to be called from the database thread
- * only!
- **/
-static void
-afsql_dd_suspend(AFSqlDestDriver *self)
-{
-  self->db_thread_suspended = TRUE;
-  g_get_current_time(&self->db_thread_suspend_target);
-  g_time_val_add(&self->db_thread_suspend_target, self->time_reopen * 1000 * 1000); /* the timeout expects microseconds */
-}
-
-static void
-afsql_dd_disconnect(AFSqlDestDriver *self)
-{
-  dbi_conn_close(self->dbi_ctx);
-  self->dbi_ctx = NULL;
-  g_hash_table_remove_all(self->syslogng_conform_tables);
-}
-
 static void
 afsql_dd_set_dbd_opt(gpointer key, gpointer value, gpointer user_data)
 {
@@ -690,11 +636,42 @@ afsql_dd_set_dbd_opt_numeric(gpointer key, gpointer value, gpointer user_data)
                               GPOINTER_TO_INT(value));
 }
 
-static gboolean
-afsql_dd_ensure_initialized_connection(AFSqlDestDriver *self)
+/*
+ * NOTE: there's a bug in libdbd-sqlite3 and this function basically works
+ * that around.  The issue is that the database path cannot be an empty
+ * string as it causes an invalid read (it is using -1 as an index in that
+ * case).  What we do is that if database is a fully specified path to a
+ * filename (e.g.  starts with a slash), we use another slash, so it remains
+ * a root-relative filename.  Otherwise, we simply use the current
+ * directory.
+ */
+static const gchar *
+_sqlite_db_dir(const gchar *database, gchar *buf, gsize buflen)
 {
-  if (self->dbi_ctx)
-    return TRUE;
+  if (database[0] == '/')
+    return strncpy(buf, "/", buflen);
+  else
+    return getcwd(buf, buflen);
+  return buf;
+}
+
+static void
+_enable_database_specific_hacks(AFSqlDestDriver *self)
+{
+  gchar buf[1024];
+
+  if (strcmp(self->type, "sqlite") == 0)
+    dbi_conn_set_option(self->dbi_ctx, "sqlite_dbdir", _sqlite_db_dir(self->database, buf, sizeof(buf)));
+  else if (strcmp(self->type, "sqlite3") == 0)
+    dbi_conn_set_option(self->dbi_ctx, "sqlite3_dbdir", _sqlite_db_dir(self->database, buf, sizeof(buf)));
+  else if (strcmp(self->type, s_oracle) == 0)
+    dbi_conn_set_option_numeric(self->dbi_ctx, "oracle_ignore_tns_config", self->ignore_tns_config);
+}
+
+static gboolean
+afsql_dd_connect(LogThreadedDestDriver *s)
+{
+  AFSqlDestDriver *self = (AFSqlDestDriver *) s;
 
   self->dbi_ctx = dbi_conn_new_r(self->type, dbi_instance);
 
@@ -718,9 +695,7 @@ afsql_dd_ensure_initialized_connection(AFSqlDestDriver *self)
   dbi_conn_set_option(self->dbi_ctx, "encoding", self->encoding);
   dbi_conn_set_option(self->dbi_ctx, "auto-commit", self->flags & AFSQL_DDF_EXPLICIT_COMMITS ? "false" : "true");
 
-  /* database specific hacks */
-  dbi_conn_set_option(self->dbi_ctx, "sqlite_dbdir", "");
-  dbi_conn_set_option(self->dbi_ctx, "sqlite3_dbdir", "");
+  _enable_database_specific_hacks(self);
 
   /* Set user-specified options */
   g_hash_table_foreach(self->dbd_options, afsql_dd_set_dbd_opt, self->dbi_ctx);
@@ -762,6 +737,15 @@ afsql_dd_ensure_initialized_connection(AFSqlDestDriver *self)
   return TRUE;
 }
 
+static void
+afsql_dd_disconnect(LogThreadedDestDriver *s)
+{
+  AFSqlDestDriver *self = (AFSqlDestDriver *) s;
+
+  dbi_conn_close(self->dbi_ctx);
+  self->dbi_ctx = NULL;
+}
+
 static GString *
 afsql_dd_ensure_accessible_database_table(AFSqlDestDriver *self, LogMessage *msg)
 {
@@ -773,7 +757,7 @@ afsql_dd_ensure_accessible_database_table(AFSqlDestDriver *self, LogMessage *msg
     {
       /* If validate table is FALSE then close the connection and wait time_reopen time (next call) */
       msg_error("Error checking table, disconnecting from database, trying again shortly",
-                evt_tag_int("time_reopen", self->time_reopen));
+                evt_tag_int("time_reopen", self->super.time_reopen));
       g_string_free(table, TRUE);
       return NULL;
     }
@@ -813,7 +797,9 @@ afsql_dd_build_insert_command(AFSqlDestDriver *self, LogMessage *msg, GString *t
 
       if ((self->fields[i].flags & AFSQL_FF_DEFAULT) == 0 && self->fields[i].value != NULL)
         {
-          log_template_format(self->fields[i].value, msg, &self->template_options, LTZ_SEND, self->seq_num, NULL, value);
+          log_template_format(self->fields[i].value, msg,
+                              &self->template_options, LTZ_SEND, self->super.worker.instance.seq_num,
+                              NULL, value);
           if (self->null_value && strcmp(self->null_value, value->str) == 0)
             {
               g_string_append(insert_command, "NULL");
@@ -850,53 +836,48 @@ afsql_dd_build_insert_command(AFSqlDestDriver *self, LogMessage *msg, GString *t
 static inline gboolean
 afsql_dd_is_transaction_handling_enabled(const AFSqlDestDriver *self)
 {
-  return self->flush_lines_queued != -1;
+  return !!(self->flags & AFSQL_DDF_EXPLICIT_COMMITS);
 }
 
 static inline gboolean
 afsql_dd_should_begin_new_transaction(const AFSqlDestDriver *self)
 {
-  return self->flush_lines_queued == 0;
+  return afsql_dd_is_transaction_handling_enabled(self) && self->super.worker.instance.batch_size == 1;
+}
+
+static gint
+_batch_size(const AFSqlDestDriver *self)
+{
+  if (self->super.batch_lines <= 0)
+    return DEFAULT_SQL_TX_SIZE;
+
+  return self->super.batch_lines;
 }
 
 static inline gboolean
 afsql_dd_should_commit_transaction(const AFSqlDestDriver *self)
 {
-  return afsql_dd_is_transaction_handling_enabled(self) && self->flush_lines_queued == self->flush_lines;
+  return afsql_dd_is_transaction_handling_enabled(self) && self->super.worker.instance.batch_size >= _batch_size(self);
 }
 
-static inline void
-afsql_dd_rollback_msg(AFSqlDestDriver *self, LogMessage *msg, LogPathOptions *path_options)
-{
-  if (self->flags & AFSQL_DDF_EXPLICIT_COMMITS)
-    log_queue_rewind_backlog(self->queue, 1);
-  else
-    log_queue_push_head(self->queue, msg, path_options);
-}
-
-static inline gboolean
-afsql_dd_handle_insert_row_error_depending_on_connection_availability(AFSqlDestDriver *self,
-    LogMessage *msg,
-    LogPathOptions *path_options)
+static worker_insert_result_t
+afsql_dd_handle_insert_row_error_depending_on_connection_availability(AFSqlDestDriver *self)
 {
   const gchar *dbi_error, *error_message;
 
   if (dbi_conn_ping(self->dbi_ctx) == 1)
     {
-      afsql_dd_rollback_msg(self, msg, path_options);
-      return TRUE;
+      return WORKER_INSERT_RESULT_ERROR;
     }
 
   if (afsql_dd_is_transaction_handling_enabled(self))
     {
       error_message = "SQL connection lost in the middle of a transaction,"
                       " rewinding backlog and starting again";
-      afsql_dd_handle_transaction_error(self);
     }
   else
     {
       error_message = "Error, no SQL connection after failed query attempt";
-      afsql_dd_rollback_msg(self, msg, path_options);
     }
 
   dbi_conn_error(self->dbi_ctx, &dbi_error);
@@ -908,7 +889,35 @@ afsql_dd_handle_insert_row_error_depending_on_connection_availability(AFSqlDestD
             evt_tag_str("database", self->database),
             evt_tag_str("error", dbi_error));
 
-  return FALSE;
+  return WORKER_INSERT_RESULT_ERROR;
+}
+
+static worker_insert_result_t
+afsql_dd_flush(LogThreadedDestDriver *s)
+{
+  AFSqlDestDriver *self = (AFSqlDestDriver *) s;
+
+  if (!afsql_dd_is_transaction_handling_enabled(self))
+    return WORKER_INSERT_RESULT_SUCCESS;
+
+  if (!afsql_dd_commit_transaction(self))
+    {
+      /* Assuming that in case of error, the queue is rewound by afsql_dd_commit_transaction() */
+      afsql_dd_rollback_transaction(self);
+      return WORKER_INSERT_RESULT_ERROR;
+    }
+  return WORKER_INSERT_RESULT_SUCCESS;
+}
+
+static gboolean
+afsql_dd_run_insert_query(AFSqlDestDriver *self, GString *table, LogMessage *msg)
+{
+  GString *insert_command;
+
+  insert_command = afsql_dd_build_insert_command(self, msg, table);
+  gboolean success = afsql_dd_run_query(self, insert_command->str, FALSE, NULL);
+  g_string_free(insert_command, TRUE);
+  return success;
 }
 
 /**
@@ -919,223 +928,49 @@ afsql_dd_handle_insert_row_error_depending_on_connection_availability(AFSqlDestD
  * Returns: FALSE to indicate that the connection should be closed and
  * this destination suspended for time_reopen() time.
  **/
-static gboolean
-afsql_dd_insert_db(AFSqlDestDriver *self)
+static worker_insert_result_t
+afsql_dd_insert(LogThreadedDestDriver *s, LogMessage *msg)
 {
+  AFSqlDestDriver *self = (AFSqlDestDriver *) s;
   GString *table = NULL;
-  GString *insert_command = NULL;
-  LogMessage *msg;
-  gboolean success = TRUE;
-  LogPathOptions path_options = LOG_PATH_OPTIONS_INIT;
-
-  if (!afsql_dd_ensure_initialized_connection(self))
-    return FALSE;
-
-  /* connection established, try to insert a message */
-  msg = log_queue_pop_head(self->queue, &path_options);
-  if (!msg)
-    return TRUE;
-
-  msg_set_context(msg);
+  worker_insert_result_t retval = WORKER_INSERT_RESULT_ERROR;
 
   table = afsql_dd_ensure_accessible_database_table(self, msg);
-
   if (!table)
-    {
-      success = FALSE;
-      goto out;
-    }
+    goto error;
 
   if (afsql_dd_should_begin_new_transaction(self) && !afsql_dd_begin_transaction(self))
+    goto error;
+
+  if (!afsql_dd_run_insert_query(self, table, msg))
     {
-      success = FALSE;
-      goto out;
+      retval = afsql_dd_handle_insert_row_error_depending_on_connection_availability(self);
+      goto error;
     }
 
-  insert_command = afsql_dd_build_insert_command(self, msg, table);
-  success = afsql_dd_run_query(self, insert_command->str, FALSE, NULL);
-
-  if (success && self->flush_lines_queued != -1)
+  if (afsql_dd_should_commit_transaction(self))
     {
-      self->flush_lines_queued++;
-
-      if (afsql_dd_should_commit_transaction(self) && !afsql_dd_commit_transaction(self))
-        {
-          /* Assuming that in case of error, the queue is rewound by afsql_dd_commit_transaction() */
-          afsql_dd_rollback_transaction(self);
-
-          g_string_free(insert_command, TRUE);
-          msg_set_context(NULL);
-
-          success = FALSE;
-        }
+      retval = afsql_dd_flush(s);
+    }
+  else
+    {
+      retval = afsql_dd_is_transaction_handling_enabled(self)
+               ? WORKER_INSERT_RESULT_QUEUED
+               : WORKER_INSERT_RESULT_SUCCESS;
     }
 
-out:
+error:
 
   if (table != NULL)
     g_string_free(table, TRUE);
 
-  if (insert_command != NULL)
-    g_string_free(insert_command, TRUE);
-
-  msg_set_context(NULL);
-
-  if (success)
-    {
-      log_msg_ack(msg, &path_options, AT_PROCESSED);
-      log_msg_unref(msg);
-      step_sequence_number(&self->seq_num);
-      self->failed_message_counter = 0;
-    }
-  else
-    {
-      if (self->failed_message_counter < self->num_retries)
-        {
-          if (!afsql_dd_handle_insert_row_error_depending_on_connection_availability(self, msg, &path_options))
-            return FALSE;
-
-          self->failed_message_counter++;
-        }
-      else
-        {
-          msg_error("Multiple failures while inserting this record into the database, message dropped",
-                    evt_tag_int("attempts", self->num_retries));
-          stats_counter_inc(self->dropped_messages);
-          log_msg_drop(msg, &path_options, AT_PROCESSED);
-          self->failed_message_counter = 0;
-          success = TRUE;
-        }
-    }
-
-  return success;
+  return retval;
 }
 
-static void
-afsql_dd_message_became_available_in_the_queue(gpointer user_data)
-{
-  AFSqlDestDriver *self = (AFSqlDestDriver *) user_data;
-
-  g_mutex_lock(self->db_thread_mutex);
-  g_cond_signal(self->db_thread_wakeup_cond);
-  g_mutex_unlock(self->db_thread_mutex);
-}
-
-/* assumes that db_thread_mutex is held */
-static void
-afsql_dd_wait_for_suspension_wakeup(AFSqlDestDriver *self)
-{
-  /* we got suspended, probably because of a connection error,
-   * during this time we only get wakeups if we need to be
-   * terminated. */
-  if (!self->db_thread_terminate)
-    g_cond_timed_wait(self->db_thread_wakeup_cond, self->db_thread_mutex, &self->db_thread_suspend_target);
-  self->db_thread_suspended = FALSE;
-}
-
-/**
- * afsql_dd_database_thread:
- *
- * This is the thread inserting records into the database.
- **/
-static void
-afsql_dd_database_thread(gpointer arg)
-{
-  AFSqlDestDriver *self = (AFSqlDestDriver *) arg;
-
-  msg_verbose("Database thread started",
-              evt_tag_str("driver", self->super.super.id));
-  while (!self->db_thread_terminate)
-    {
-      main_loop_worker_run_gc();
-      g_mutex_lock(self->db_thread_mutex);
-      if (self->db_thread_suspended)
-        {
-          afsql_dd_wait_for_suspension_wakeup(self);
-          /* we loop back to check if the thread was requested to terminate */
-        }
-      else if (!log_queue_check_items(self->queue, NULL, afsql_dd_message_became_available_in_the_queue, self, NULL))
-        {
-          /* we have nothing to INSERT into the database, let's wait we get some new stuff */
-
-          if (self->flush_lines_queued > 0)
-            {
-              if (!afsql_dd_commit_transaction(self))
-                {
-                  if (!afsql_dd_rollback_transaction(self))
-                    {
-                      afsql_dd_disconnect(self);
-                      afsql_dd_suspend(self);
-                    }
-                  g_mutex_unlock(self->db_thread_mutex);
-                  continue;
-                }
-            }
-          else if (!self->db_thread_terminate)
-            {
-              g_cond_wait(self->db_thread_wakeup_cond, self->db_thread_mutex);
-            }
-
-          /* we loop back to check if the thread was requested to terminate */
-        }
-      g_mutex_unlock(self->db_thread_mutex);
-
-      if (self->db_thread_terminate)
-        break;
-
-      if (!afsql_dd_insert_db(self))
-        {
-          afsql_dd_disconnect(self);
-          afsql_dd_suspend(self);
-        }
-    }
-
-  while (log_queue_get_length(self->queue) > 0)
-    {
-      if (!afsql_dd_insert_db(self))
-        {
-          goto exit;
-        }
-    }
-
-  if (self->flush_lines_queued > 0)
-    {
-      /* we can't do anything with the return value here. if commit isn't
-       * successful, we get our backlog back, but we have no chance
-       * submitting that back to the SQL engine.
-       */
-
-      if (!afsql_dd_commit_transaction(self))
-        afsql_dd_rollback_transaction(self);
-    }
-exit:
-  afsql_dd_disconnect(self);
-
-  msg_verbose("Database thread finished",
-              evt_tag_str("driver", self->super.super.id));
-}
-
-static void
-afsql_dd_stop_thread(gpointer s)
+static const gchar *
+afsql_dd_format_stats_instance(LogThreadedDestDriver *s)
 {
   AFSqlDestDriver *self = (AFSqlDestDriver *) s;
-  g_mutex_lock(self->db_thread_mutex);
-  self->db_thread_terminate = TRUE;
-  g_cond_signal(self->db_thread_wakeup_cond);
-  g_mutex_unlock(self->db_thread_mutex);
-}
-
-static void
-afsql_dd_start_thread(AFSqlDestDriver *self)
-{
-  self->db_thread_terminate = FALSE;
-  main_loop_create_worker_thread(afsql_dd_database_thread, afsql_dd_stop_thread, self, &self->worker_options);
-}
-
-
-static gchar *
-afsql_dd_format_stats_instance(AFSqlDestDriver *self)
-{
   static gchar persist_name[64];
 
   g_snprintf(persist_name, sizeof(persist_name),
@@ -1144,7 +979,7 @@ afsql_dd_format_stats_instance(AFSqlDestDriver *self)
   return persist_name;
 }
 
-static inline const gchar *
+static const gchar *
 afsql_dd_format_persist_name(const LogPipe *s)
 {
   AFSqlDestDriver *self = (AFSqlDestDriver *)s;
@@ -1159,138 +994,80 @@ afsql_dd_format_persist_name(const LogPipe *s)
   return persist_name;
 }
 
-static inline gchar *
-afsql_dd_format_persist_sequence_number(AFSqlDestDriver *self)
+static gboolean
+_init_fields_from_columns_and_values(AFSqlDestDriver *self)
 {
-  static gchar persist_name[256];
+  GlobalConfig *cfg = log_pipe_get_config(&self->super.super.super.super);
+  GList *col, *value;
+  gint len_cols, len_values;
+  gint i;
 
-  g_snprintf(persist_name, sizeof(persist_name),
-             "afsql_dd_sequence_number(%s,%s,%s,%s,%s)",
-             self->type,self->host, self->port, self->database, self->table->template);
+  if (self->fields)
+    return TRUE;
 
-  return persist_name;
+  len_cols = g_list_length(self->columns);
+  len_values = g_list_length(self->values);
+  if (len_cols != len_values)
+    {
+      msg_error("The number of columns and values do not match",
+                evt_tag_int("len_columns", len_cols),
+                evt_tag_int("len_values", len_values));
+      return FALSE;
+    }
+  self->fields_len = len_cols;
+  self->fields = g_new0(AFSqlField, len_cols);
+
+  for (i = 0, col = self->columns, value = self->values; col && value; i++, col = col->next, value = value->next)
+    {
+      gchar *space;
+
+      space = strchr(col->data, ' ');
+      if (space)
+        {
+          self->fields[i].name = g_strndup(col->data, space - (gchar *) col->data);
+          while (*space == ' ')
+            space++;
+          if (*space != '\0')
+            self->fields[i].type = g_strdup(space);
+          else
+            self->fields[i].type = g_strdup("text");
+        }
+      else
+        {
+          self->fields[i].name = g_strdup(col->data);
+          self->fields[i].type = g_strdup("text");
+        }
+      if (!_is_sql_identifier_sanitized(self->fields[i].name))
+        {
+          msg_error("Column name is not a proper SQL name",
+                    evt_tag_str("column", self->fields[i].name));
+          return FALSE;
+        }
+
+      if (GPOINTER_TO_UINT(value->data) > 4096)
+        {
+          self->fields[i].value = log_template_new(cfg, NULL);
+          log_template_compile(self->fields[i].value, (gchar *) value->data, NULL);
+        }
+      else
+        {
+          switch (GPOINTER_TO_UINT(value->data))
+            {
+            case AFSQL_COLUMN_DEFAULT:
+              self->fields[i].flags |= AFSQL_FF_DEFAULT;
+              break;
+            default:
+              g_assert_not_reached();
+              break;
+            }
+        }
+    }
+  return TRUE;
 }
 
 static gboolean
-afsql_dd_init(LogPipe *s)
+_initialize_dbi(void)
 {
-  AFSqlDestDriver *self = (AFSqlDestDriver *) s;
-  GlobalConfig *cfg = log_pipe_get_config(s);
-  gint len_cols, len_values;
-
-  if (!log_dest_driver_init_method(s))
-    return FALSE;
-
-  if (!self->columns || !self->values)
-    {
-      msg_error("Default columns and values must be specified for database destinations",
-                evt_tag_str("database type", self->type));
-      return FALSE;
-    }
-
-  stats_lock();
-  {
-    StatsClusterKey sc_key;
-    stats_cluster_logpipe_key_set(&sc_key, SCS_SQL | SCS_DESTINATION, self->super.super.id,
-                                  afsql_dd_format_stats_instance(self) );
-    stats_register_counter(0, &sc_key, SC_TYPE_QUEUED, &self->queued_messages);
-    stats_register_counter(0, &sc_key, SC_TYPE_DROPPED, &self->dropped_messages);
-    stats_register_counter(STATS_LEVEL1, &sc_key, SC_TYPE_MEMORY_USAGE, &self->memory_usage);
-  }
-  stats_unlock();
-
-  self->seq_num = GPOINTER_TO_INT(cfg_persist_config_fetch(cfg, afsql_dd_format_persist_sequence_number(self)));
-  if (!self->seq_num)
-    init_sequence_number(&self->seq_num);
-
-  self->queue = log_dest_driver_acquire_queue(&self->super,
-                                              afsql_dd_format_persist_name((const LogPipe *)self));
-  if (self->queue == NULL)
-    {
-      return FALSE;
-    }
-  else
-    {
-      if (self->flags & AFSQL_DDF_EXPLICIT_COMMITS)
-        log_queue_set_use_backlog(self->queue, TRUE);
-    }
-  log_queue_set_counters(self->queue, self->queued_messages, self->dropped_messages, self->memory_usage);
-  if (!self->fields)
-    {
-      GList *col, *value;
-      gint i;
-
-      len_cols = g_list_length(self->columns);
-      len_values = g_list_length(self->values);
-      if (len_cols != len_values)
-        {
-          msg_error("The number of columns and values do not match",
-                    evt_tag_int("len_columns", len_cols),
-                    evt_tag_int("len_values", len_values));
-          goto error;
-        }
-      self->fields_len = len_cols;
-      self->fields = g_new0(AFSqlField, len_cols);
-
-      for (i = 0, col = self->columns, value = self->values; col && value; i++, col = col->next, value = value->next)
-        {
-          gchar *space;
-
-          space = strchr(col->data, ' ');
-          if (space)
-            {
-              self->fields[i].name = g_strndup(col->data, space - (gchar *) col->data);
-              while (*space == ' ')
-                space++;
-              if (*space != '\0')
-                self->fields[i].type = g_strdup(space);
-              else
-                self->fields[i].type = g_strdup("text");
-            }
-          else
-            {
-              self->fields[i].name = g_strdup(col->data);
-              self->fields[i].type = g_strdup("text");
-            }
-          if (!_is_sql_identifier_sanitized(self->fields[i].name))
-            {
-              msg_error("Column name is not a proper SQL name",
-                        evt_tag_str("column", self->fields[i].name));
-              return FALSE;
-            }
-
-          if (GPOINTER_TO_UINT(value->data) > 4096)
-            {
-              self->fields[i].value = log_template_new(cfg, NULL);
-              log_template_compile(self->fields[i].value, (gchar *) value->data, NULL);
-            }
-          else
-            {
-              switch (GPOINTER_TO_UINT(value->data))
-                {
-                case AFSQL_COLUMN_DEFAULT:
-                  self->fields[i].flags |= AFSQL_FF_DEFAULT;
-                  break;
-                default:
-                  g_assert_not_reached();
-                  break;
-                }
-            }
-        }
-    }
-
-  self->time_reopen = cfg->time_reopen;
-
-  log_template_options_init(&self->template_options, cfg);
-
-  if (self->flush_lines == -1)
-    self->flush_lines = cfg->flush_lines;
-  if (self->flush_timeout == -1)
-    self->flush_timeout = cfg->flush_timeout;
-
-  if ((self->flags & AFSQL_DDF_EXPLICIT_COMMITS) && (self->flush_lines > 0 || self->flush_timeout > 0))
-    self->flush_lines_queued = 0;
-
   if (!dbi_initialized)
     {
       errno = 0;
@@ -1301,76 +1078,51 @@ afsql_dd_init(LogPipe *s)
           /* NOTE: errno might be unreliable, but that's all we have */
           msg_error("Unable to initialize database access (DBI)",
                     evt_tag_int("rc", rc),
-                    evt_tag_errno("error", errno));
-          goto error;
+                    evt_tag_error("error"));
+          return FALSE;
         }
       else if (rc == 0)
         {
           msg_error("The database access library (DBI) reports no usable SQL drivers, perhaps DBI drivers are not installed properly");
-          goto error;
+          return FALSE;
         }
       else
         {
           dbi_initialized = TRUE;
         }
     }
-
-  afsql_dd_start_thread(self);
   return TRUE;
-
-error:
-
-  stats_lock();
-  {
-    StatsClusterKey sc_key;
-    stats_cluster_logpipe_key_set(&sc_key, SCS_SQL | SCS_DESTINATION, self->super.super.id,
-                                  afsql_dd_format_stats_instance(self) );
-    stats_unregister_counter(&sc_key, SC_TYPE_QUEUED, &self->queued_messages);
-    stats_unregister_counter(&sc_key, SC_TYPE_DROPPED, &self->dropped_messages);
-    stats_unregister_counter(&sc_key, SC_TYPE_MEMORY_USAGE, &self->memory_usage);
-  }
-  stats_unlock();
-
-  return FALSE;
 }
 
 static gboolean
-afsql_dd_deinit(LogPipe *s)
+afsql_dd_init(LogPipe *s)
 {
   AFSqlDestDriver *self = (AFSqlDestDriver *) s;
+  GlobalConfig *cfg = log_pipe_get_config(s);
 
-  log_queue_reset_parallel_push(self->queue);
-
-  log_queue_set_counters(self->queue, NULL, NULL, NULL);
-  cfg_persist_config_add(log_pipe_get_config(s), afsql_dd_format_persist_sequence_number(self),
-                         GINT_TO_POINTER(self->seq_num), NULL, FALSE);
-
-  stats_lock();
-  StatsClusterKey sc_key;
-  stats_cluster_logpipe_key_set(&sc_key, SCS_SQL | SCS_DESTINATION, self->super.super.id,
-                                afsql_dd_format_stats_instance(self) );
-  stats_unregister_counter(&sc_key, SC_TYPE_QUEUED, &self->queued_messages);
-  stats_unregister_counter(&sc_key, SC_TYPE_DROPPED, &self->dropped_messages);
-  stats_unlock();
-
-  if (!log_dest_driver_deinit_method(s))
+  if (!log_threaded_dest_driver_init_method(s))
+    return FALSE;
+  if (!_initialize_dbi())
     return FALSE;
 
-  return TRUE;
-}
+  if (!self->columns || !self->values)
+    {
+      msg_error("Default columns and values must be specified for database destinations",
+                evt_tag_str("type", self->type));
+      return FALSE;
+    }
 
-static void
-afsql_dd_queue(LogPipe *s, LogMessage *msg, const LogPathOptions *path_options, gpointer user_data)
-{
-  AFSqlDestDriver *self = (AFSqlDestDriver *) s;
-  LogPathOptions local_options;
+  if (self->ignore_tns_config && strcmp(self->type, s_oracle) != 0)
+    {
+      msg_warning("WARNING: Option ignore_tns_config was skipped because database type is not Oracle",
+                  evt_tag_str("type", self->type));
+    }
 
-  if (!path_options->flow_control_requested)
-    path_options = log_msg_break_ack(msg, path_options, &local_options);
+  if (!_init_fields_from_columns_and_values(self))
+    return FALSE;
 
-  log_msg_add_ack(msg, path_options);
-  log_queue_push_tail(self->queue, log_msg_ref(msg), path_options);
-  log_dest_driver_queue_method(s, msg, path_options, user_data);
+  log_template_options_init(&self->template_options, cfg);
+  return log_threaded_dest_driver_start_workers(&self->super);
 }
 
 static void
@@ -1407,9 +1159,7 @@ afsql_dd_free(LogPipe *s)
   g_hash_table_destroy(self->dbd_options_numeric);
   if (self->session_statements)
     string_list_free(self->session_statements);
-  g_mutex_free(self->db_thread_mutex);
-  g_cond_free(self->db_thread_wakeup_cond);
-  log_dest_driver_free(s);
+  log_threaded_dest_driver_free(s);
 }
 
 LogDriver *
@@ -1417,13 +1167,16 @@ afsql_dd_new(GlobalConfig *cfg)
 {
   AFSqlDestDriver *self = g_new0(AFSqlDestDriver, 1);
 
-  log_dest_driver_init_instance(&self->super, cfg);
+  log_threaded_dest_driver_init_instance(&self->super, cfg);
 
-  self->super.super.super.init = afsql_dd_init;
-  self->super.super.super.deinit = afsql_dd_deinit;
-  self->super.super.super.queue = afsql_dd_queue;
-  self->super.super.super.free_fn = afsql_dd_free;
-  self->super.super.super.generate_persist_name = afsql_dd_format_persist_name;
+  self->super.super.super.super.init = afsql_dd_init;
+  self->super.super.super.super.free_fn = afsql_dd_free;
+  self->super.super.super.super.generate_persist_name = afsql_dd_format_persist_name;
+  self->super.format_stats_instance = afsql_dd_format_stats_instance;
+  self->super.worker.connect = afsql_dd_connect;
+  self->super.worker.disconnect = afsql_dd_disconnect;
+  self->super.worker.insert = afsql_dd_insert;
+  self->super.worker.flush = afsql_dd_flush;
 
   self->type = g_strdup("mysql");
   self->host = g_strdup("");
@@ -1433,36 +1186,30 @@ afsql_dd_new(GlobalConfig *cfg)
   self->database = g_strdup("logs");
   self->encoding = g_strdup("UTF-8");
   self->transaction_active = FALSE;
+  self->ignore_tns_config = FALSE;
 
   self->table = log_template_new(configuration, NULL);
   log_template_compile(self->table, "messages", NULL);
   self->failed_message_counter = 0;
 
-  self->flush_lines = -1;
-  self->flush_timeout = -1;
-  self->flush_lines_queued = -1;
   self->session_statements = NULL;
-  self->num_retries = MAX_FAILED_ATTEMPTS;
 
   self->syslogng_conform_tables = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, NULL);
   self->dbd_options = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
   self->dbd_options_numeric = g_hash_table_new_full(g_str_hash, g_int_equal, g_free, NULL);
 
   log_template_options_defaults(&self->template_options);
+  self->super.stats_source = SCS_SQL;
 
-  self->db_thread_wakeup_cond = g_cond_new();
-  self->db_thread_mutex = g_mutex_new();
-
-  self->worker_options.is_output_thread = TRUE;
-  return &self->super.super;
+  return &self->super.super.super;
 }
 
 gint
 afsql_dd_lookup_flag(const gchar *flag)
 {
-  if (strcmp(flag, "explicit-commits") == 0 || strcmp(flag, "explicit_commits") == 0)
+  if (strcmp(flag, "explicit-commits") == 0)
     return AFSQL_DDF_EXPLICIT_COMMITS;
-  else if (strcmp(flag, "dont-create-tables") == 0 || strcmp(flag, "dont_create_tables") == 0)
+  else if (strcmp(flag, "dont-create-tables") == 0)
     return AFSQL_DDF_DONT_CREATE_TABLES;
   else
     msg_warning("Unknown SQL flag",
