@@ -29,6 +29,7 @@
 #include "parse-number.h"
 #include "str-format.h"
 #include "plugin-types.h"
+#include "scratch-buffers.h"
 
 #include <stdlib.h>
 #include <errno.h>
@@ -38,6 +39,7 @@
  * include them all here. If it causes compilation times to increase
  * drastically, we should probably make them into separate compilation
  * units. (Bazsi) */
+#include "urlencode.c"
 #include "numeric-funcs.c"
 #include "str-funcs.c"
 #include "cond-funcs.c"
@@ -102,7 +104,10 @@ static Plugin basicfuncs_plugins[] =
 
   /* misc funcs */
   TEMPLATE_FUNCTION_PLUGIN(tf_env, "env"),
-  TEMPLATE_FUNCTION_PLUGIN(tf_template, "template")
+  TEMPLATE_FUNCTION_PLUGIN(tf_template, "template"),
+  TEMPLATE_FUNCTION_PLUGIN(tf_urlencode, "url-encode"),
+  TEMPLATE_FUNCTION_PLUGIN(tf_urldecode, "url-decode"),
+  TEMPLATE_FUNCTION_PLUGIN(tf_base64encode, "base64-encode")
 };
 
 gboolean
