@@ -140,6 +140,9 @@ dqtool_cat(int argc, char *argv[])
       if (!open_queue(argv[i], &lq, &options))
         continue;
 
+      log_queue_set_use_backlog(lq, TRUE);
+      log_queue_rewind_backlog_all(lq);
+
       while ((log_msg = log_queue_pop_head(lq, &local_options)) != NULL)
         {
           /* format log */
@@ -232,7 +235,6 @@ usage(void)
     {
       fprintf(stderr, "    %-12s %s\n", modes[mode].mode, modes[mode].description);
     }
-  exit(1);
 }
 
 void
@@ -253,6 +255,7 @@ main(int argc, char *argv[])
   if (!mode_string)
     {
       usage();
+      return 1;
     }
 
   ctx = NULL;
@@ -274,6 +277,7 @@ main(int argc, char *argv[])
     {
       fprintf(stderr, "Unknown command\n");
       usage();
+      return 1;
     }
 
   if (!g_option_context_parse(ctx, &argc, &argv, &error))
