@@ -36,14 +36,14 @@ tf_getent_group(gchar *key, gchar *member_name, GString *result)
   char *buf;
   long bufsize;
   int s;
-  glong d;
+  gint64 d;
   gboolean is_num, r;
 
   bufsize = 16384;
 
   buf = g_malloc(bufsize);
 
-  if ((is_num = parse_number(key, &d)) == TRUE)
+  if ((is_num = parse_dec_number(key, &d)) == TRUE)
     s = getgrgid_r((gid_t)d, &grp, buf, bufsize, &res);
   else
     s = getgrnam_r(key, &grp, buf, bufsize, &res);
@@ -52,8 +52,7 @@ tf_getent_group(gchar *key, gchar *member_name, GString *result)
     {
       msg_error("$(getent group) failed",
                 evt_tag_str("key", key),
-                evt_tag_errno("errno", errno),
-                NULL);
+                evt_tag_error("errno"));
       g_free(buf);
       return FALSE;
     }
@@ -78,8 +77,7 @@ tf_getent_group(gchar *key, gchar *member_name, GString *result)
     {
       msg_error("$(getent group): unknown member",
                 evt_tag_str("key", key),
-                evt_tag_str("member", member_name),
-                NULL);
+                evt_tag_str("member", member_name));
       g_free(buf);
       return FALSE;
     }
