@@ -20,285 +20,19 @@
  *
  */
 
-#include <curl/curl.h>
+#include "http.h"
+#include "http-worker.h"
 
-#include "syslog-names.h"
-#include "http-plugin.h"
-#include "scratch-buffers.h"
-
-static const gchar *
-_format_persist_name(const LogPipe *s)
-{
-  const HTTPDestinationDriver *self = (const HTTPDestinationDriver *)s;
-  static gchar persist_name[1024];
-
-  if (s->persist_name)
-    g_snprintf(persist_name, sizeof(persist_name), "http.%s", s->persist_name);
-  else
-    g_snprintf(persist_name, sizeof(persist_name), "http(%s,)", self->url);
-
-  return persist_name;
-}
-
-static gchar *
-_format_stats_instance(LogThrDestDriver *s)
-{
-  static gchar stats[1024];
-
-  HTTPDestinationDriver *self = (HTTPDestinationDriver *) s;
-
-  g_snprintf(stats, sizeof(stats), "http,%s", self->url);
-
-  return stats;
-}
-
-static size_t
-_http_write_cb(char *ptr, size_t size, size_t nmemb, void *userdata)
-{
-  // Discard response content
-  return nmemb * size;
-}
-
-static void
-_thread_init(LogThrDestDriver *s)
-{
-  HTTPDestinationDriver *self = (HTTPDestinationDriver *) s;
-
-  curl_version_info_data *curl_info = curl_version_info(CURLVERSION_NOW);
-  if (!self->user_agent)
-    self->user_agent = g_strdup_printf("syslog-ng %s/libcurl %s",
-                                       SYSLOG_NG_VERSION, curl_info->version);
-}
-
-static void
-_thread_deinit(LogThrDestDriver *s)
-{
-}
-
-static gboolean
-_connect(LogThrDestDriver *s)
-{
-  return TRUE;
-}
-
-static void
-_disconnect(LogThrDestDriver *s)
-{
-}
-
-static void
-_add_custom_curl_header(gpointer data, gpointer curl_headers)
-{
-  curl_headers = curl_slist_append((struct curl_slist *)curl_headers, data);
-}
-
-static struct curl_slist *
-_get_curl_headers(HTTPDestinationDriver *self, LogMessage *msg)
-{
-  struct curl_slist *curl_headers = NULL;
-  gchar header_host[128] = {0};
-  gchar header_program[32] = {0};
-  gchar header_facility[32] = {0};
-  gchar header_level[32] = {0};
-
-  g_snprintf(header_host, sizeof(header_host),
-             "X-Syslog-Host: %s", log_msg_get_value(msg, LM_V_HOST, NULL));
-  curl_headers = curl_slist_append(curl_headers, header_host);
-
-  g_snprintf(header_program, sizeof(header_program),
-             "X-Syslog-Program: %s", log_msg_get_value(msg, LM_V_PROGRAM, NULL));
-  curl_headers = curl_slist_append(curl_headers, header_program);
-
-  g_snprintf(header_facility, sizeof(header_facility),
-             "X-Syslog-Facility: %s", syslog_name_lookup_name_by_value(msg->pri & LOG_FACMASK, sl_facilities));
-  curl_headers = curl_slist_append(curl_headers, header_facility);
-
-  g_snprintf(header_level, sizeof(header_level),
-             "X-Syslog-Level: %s", syslog_name_lookup_name_by_value(msg->pri & LOG_PRIMASK, sl_levels));
-  curl_headers = curl_slist_append(curl_headers, header_level);
-
-  g_list_foreach(self->headers, _add_custom_curl_header, curl_headers);
-
-  return curl_headers;
-}
-
-static const gchar *
-_get_body(HTTPDestinationDriver *self, LogMessage *msg)
-{
-  GString *body_rendered = scratch_buffers_alloc();
-
-  if (self->body_template)
-    {
-      log_template_format(self->body_template, msg, &self->template_options, LTZ_SEND,
-                          self->super.seq_num, NULL, body_rendered);
-      return body_rendered->str;
-    }
-  else
-    return log_msg_get_value(msg, LM_V_MESSAGE, NULL);
-}
-
-static
-void _http_trace_sanitize_dump(const gchar *text, gchar *data, size_t size)
-{
-  gchar *sanitized = g_new0(gchar, size+1);
-  int i;
-  for (i = 0; i < size && data[i]; i++)
-    {
-      sanitized[i] = g_ascii_isprint(data[i]) ? data[i] : '.';
-    }
-  sanitized[i] = 0;
-  msg_debug("curl trace log",
-            evt_tag_str("curl_info_type", text),
-            evt_tag_str("data", sanitized));
-  g_free(sanitized);
-}
-
-gchar *curl_infotype_to_text[] =
-{
-  "curl_trace_text",
-  "curl_trace_header_in",
-  "curl_trace_header_out",
-  "curl_trace_data_in",
-  "curl_trace_data_out",
-  "curl_trace_ssl_data_in",
-  "curl_trace_ssl_data_out",
-};
-
-static
-gint _http_trace(CURL *handle, curl_infotype type,
-                 char *data, size_t size,
-                 void *userp)
-{
-  if (!G_UNLIKELY(debug_flag))
-    return 0;
-
-  g_assert(type < sizeof(curl_infotype_to_text)/sizeof(curl_infotype_to_text[0]));
-
-  _http_trace_sanitize_dump(curl_infotype_to_text[type], data, size);
-
-  return 0;
-}
-
-static void
-_set_curl_opt(HTTPDestinationDriver *self)
-{
-  curl_easy_reset(self->curl);
-
-  curl_easy_setopt(self->curl, CURLOPT_WRITEFUNCTION, _http_write_cb);
-
-  curl_easy_setopt(self->curl, CURLOPT_URL, self->url);
-
-  if (self->user)
-    curl_easy_setopt(self->curl, CURLOPT_USERNAME, self->user);
-
-  if (self->password)
-    curl_easy_setopt(self->curl, CURLOPT_PASSWORD, self->password);
-
-  if (self->user_agent)
-    curl_easy_setopt(self->curl, CURLOPT_USERAGENT, self->user_agent);
-
-  if (self->ca_dir)
-    curl_easy_setopt(self->curl, CURLOPT_CAPATH, self->ca_dir);
-
-  if (self->ca_file)
-    curl_easy_setopt(self->curl, CURLOPT_CAINFO, self->ca_file);
-
-  if (self->cert_file)
-    curl_easy_setopt(self->curl, CURLOPT_SSLCERT, self->cert_file);
-
-  if (self->key_file)
-    curl_easy_setopt(self->curl, CURLOPT_SSLKEY, self->key_file);
-
-  if (self->ciphers)
-    curl_easy_setopt(self->curl, CURLOPT_SSL_CIPHER_LIST, self->ciphers);
-
-  curl_easy_setopt(self->curl, CURLOPT_SSLVERSION, self->ssl_version);
-
-  curl_easy_setopt(self->curl, CURLOPT_SSL_VERIFYHOST, self->peer_verify ? 2L : 0L);
-  curl_easy_setopt(self->curl, CURLOPT_SSL_VERIFYPEER, self->peer_verify ? 1L : 0L);
-
-  curl_easy_setopt(self->curl, CURLOPT_DEBUGFUNCTION, _http_trace);
-  curl_easy_setopt(self->curl, CURLOPT_VERBOSE, 1L);
-
-  curl_easy_setopt(self->curl, CURLOPT_TIMEOUT, self->timeout);
-
-  if (self->method_type == METHOD_TYPE_PUT)
-    curl_easy_setopt(self->curl, CURLOPT_CUSTOMREQUEST, "PUT");
-}
-
-static void
-_set_payload(HTTPDestinationDriver *self, struct curl_slist *curl_headers, const gchar *body)
-{
-  curl_easy_setopt(self->curl, CURLOPT_HTTPHEADER, curl_headers);
-  curl_easy_setopt(self->curl, CURLOPT_POSTFIELDS, body);
-}
-
-static worker_insert_result_t
-_map_http_status_to_worker_status(glong http_code)
-{
-  worker_insert_result_t retval;
-
-  switch (http_code/100)
-    {
-    case 4:
-      msg_debug("curl: 4XX: msg dropped",
-                evt_tag_int("status_code", http_code));
-      retval = WORKER_INSERT_RESULT_DROP;
-      break;
-    case 5:
-      msg_debug("curl: 5XX: message will be retried",
-                evt_tag_int("status_code", http_code));
-      retval = WORKER_INSERT_RESULT_ERROR;
-      break;
-    default:
-      msg_debug("curl: OK status code",
-                evt_tag_int("status_code", http_code));
-      retval = WORKER_INSERT_RESULT_SUCCESS;
-      break;
-    }
-
-  return retval;
-}
-
-static worker_insert_result_t
-_insert(LogThrDestDriver *s, LogMessage *msg)
-{
-  CURLcode ret;
-  worker_insert_result_t retval;
-
-  HTTPDestinationDriver *self = (HTTPDestinationDriver *) s;
-
-  struct curl_slist *curl_headers = _get_curl_headers(self, msg);
-  const gchar *body = _get_body(self, msg);
-  _set_payload(self, curl_headers, body);
-
-  if ((ret = curl_easy_perform(self->curl)) != CURLE_OK)
-    {
-      msg_error("curl: error sending HTTP request",
-                evt_tag_str("error", curl_easy_strerror(ret)),
-                log_pipe_location_tag(&s->super.super.super));
-
-      curl_slist_free_all(curl_headers);
-
-      return WORKER_INSERT_RESULT_NOT_CONNECTED;
-    }
-
-  glong http_code = 0;
-  curl_easy_getinfo (self->curl, CURLINFO_RESPONSE_CODE, &http_code);
-  retval = _map_http_status_to_worker_status(http_code);
-
-  curl_slist_free_all(curl_headers);
-
-  return retval;
-}
+/* HTTPDestinationDriver */
 
 void
-http_dd_set_url(LogDriver *d, const gchar *url)
+http_dd_set_urls(LogDriver *d, GList *urls)
 {
   HTTPDestinationDriver *self = (HTTPDestinationDriver *) d;
 
-  g_free(self->url);
-  self->url = g_strdup(url);
+  http_load_balancer_drop_all_targets(self->load_balancer);
+  for (GList *l = urls; l; l = l->next)
+    http_load_balancer_add_target(self->load_balancer, l->data);
 }
 
 void
@@ -363,6 +97,14 @@ http_dd_set_body(LogDriver *d, LogTemplate *body)
   self->body_template = log_template_ref(body);
 }
 
+void
+http_dd_set_delimiter(LogDriver *d, const gchar *delimiter)
+{
+  HTTPDestinationDriver *self = (HTTPDestinationDriver *) d;
+
+  g_string_assign(self->delimiter, delimiter);
+}
+
 LogTemplateOptions *
 http_dd_get_template_options(LogDriver *d)
 {
@@ -380,6 +122,14 @@ http_dd_set_ca_dir(LogDriver *d, const gchar *ca_dir)
   self->ca_dir = g_strdup(ca_dir);
 }
 
+void
+http_dd_set_use_system_cert_store(LogDriver *d, gboolean enable)
+{
+  HTTPDestinationDriver *self = (HTTPDestinationDriver *) d;
+
+  self->use_system_cert_store = enable;
+}
+
 void
 http_dd_set_ca_file(LogDriver *d, const gchar *ca_file)
 {
@@ -483,6 +233,14 @@ http_dd_set_peer_verify(LogDriver *d, gboolean verify)
   self->peer_verify = verify;
 }
 
+void
+http_dd_set_accept_redirects(LogDriver *d, gboolean accept_redirects)
+{
+  HTTPDestinationDriver *self = (HTTPDestinationDriver *) d;
+
+  self->accept_redirects = accept_redirects;
+}
+
 void
 http_dd_set_timeout(LogDriver *d, glong timeout)
 {
@@ -491,38 +249,84 @@ http_dd_set_timeout(LogDriver *d, glong timeout)
   self->timeout = timeout;
 }
 
+void
+http_dd_set_batch_bytes(LogDriver *d, glong batch_bytes)
+{
+  HTTPDestinationDriver *self = (HTTPDestinationDriver *) d;
+
+  self->batch_bytes = batch_bytes;
+}
+
+void
+http_dd_set_body_prefix(LogDriver *d, const gchar *body_prefix)
+{
+  HTTPDestinationDriver *self = (HTTPDestinationDriver *) d;
+
+  g_string_assign(self->body_prefix, body_prefix);
+}
+
+void
+http_dd_set_body_suffix(LogDriver *d, const gchar *body_suffix)
+{
+  HTTPDestinationDriver *self = (HTTPDestinationDriver *) d;
+
+  g_string_assign(self->body_suffix, body_suffix);
+}
+
+static const gchar *
+_format_persist_name(const LogPipe *s)
+{
+  const HTTPDestinationDriver *self = (const HTTPDestinationDriver *)s;
+  static gchar persist_name[1024];
+
+  if (s->persist_name)
+    g_snprintf(persist_name, sizeof(persist_name), "http.%s", s->persist_name);
+  else
+    g_snprintf(persist_name, sizeof(persist_name), "http(%s,)", self->url);
+
+  return persist_name;
+}
+
+static const gchar *
+_format_stats_instance(LogThreadedDestDriver *s)
+{
+  static gchar stats[1024];
+
+  HTTPDestinationDriver *self = (HTTPDestinationDriver *) s;
+
+  g_snprintf(stats, sizeof(stats), "http,%s", self->url);
+
+  return stats;
+}
+
 gboolean
 http_dd_init(LogPipe *s)
 {
   HTTPDestinationDriver *self = (HTTPDestinationDriver *)s;
   GlobalConfig *cfg = log_pipe_get_config(s);
 
-  if (!log_dest_driver_init_method(s))
+  if (self->load_balancer->num_targets == 0)
+    http_load_balancer_add_target(self->load_balancer, HTTP_DEFAULT_URL);
+
+  if (self->load_balancer->num_targets > 1 && s->persist_name == NULL)
+    {
+      msg_warning("WARNING: your http() driver instance uses multiple urls without persist-name(). "
+                  "It is recommended that you set persist-name() in this case as syslog-ng will be "
+                  "using the first URL in urls() to register persistent data, such as the disk queue "
+                  "name, which might change",
+                  evt_tag_str("url", self->load_balancer->targets[0].url));
+    }
+  /* we need to set up url before we call the inherited init method, so our stats key is correct */
+  self->url = self->load_balancer->targets[0].url;
+
+  if (!log_threaded_dest_driver_init_method(s))
     return FALSE;
 
   log_template_options_init(&self->template_options, cfg);
 
-  if (!(self->curl = curl_easy_init()))
-    {
-      msg_error("curl: cannot initialize libcurl",
-                log_pipe_location_tag(s));
-      return FALSE;
-    }
+  http_load_balancer_set_recovery_timeout(self->load_balancer, self->super.time_reopen);
 
-  if (!self->url)
-    {
-      self->url = g_strdup(HTTP_DEFAULT_URL);
-    }
-
-  _set_curl_opt(self);
-
-  return log_threaded_dest_driver_start(s);
-}
-
-gboolean
-http_dd_deinit(LogPipe *s)
-{
-  return log_threaded_dest_driver_deinit_method(s);
+  return log_threaded_dest_driver_start_workers(&self->super);
 }
 
 static void
@@ -532,10 +336,12 @@ http_dd_free(LogPipe *s)
 
   log_template_options_destroy(&self->template_options);
 
-  curl_easy_cleanup(self->curl);
+  g_string_free(self->delimiter, TRUE);
+  g_string_free(self->body_prefix, TRUE);
+  g_string_free(self->body_suffix, TRUE);
+
   curl_global_cleanup();
 
-  g_free(self->url);
   g_free(self->user);
   g_free(self->password);
   g_free(self->user_agent);
@@ -545,11 +351,11 @@ http_dd_free(LogPipe *s)
   g_free(self->key_file);
   g_free(self->ciphers);
   g_list_free_full(self->headers, g_free);
+  http_load_balancer_free(self->load_balancer);
 
   log_threaded_dest_driver_free(s);
 }
 
-
 LogDriver *
 http_dd_new(GlobalConfig *cfg)
 {
@@ -558,21 +364,27 @@ http_dd_new(GlobalConfig *cfg)
   log_threaded_dest_driver_init_instance(&self->super, cfg);
 
   self->super.super.super.super.init = http_dd_init;
-  self->super.super.super.super.deinit = http_dd_deinit;
-  self->super.worker.thread_init = _thread_init;
-  self->super.worker.thread_deinit = _thread_deinit;
-  self->super.worker.connect = _connect;
-  self->super.worker.disconnect = _disconnect;
-  self->super.worker.insert = _insert;
-  self->super.super.super.super.generate_persist_name = _format_persist_name;
-  self->super.format.stats_instance = _format_stats_instance;
-  self->super.stats_source = SCS_HTTP;
   self->super.super.super.super.free_fn = http_dd_free;
+  self->super.super.super.super.generate_persist_name = _format_persist_name;
+  self->super.format_stats_instance = _format_stats_instance;
+  self->super.stats_source = SCS_HTTP;
+  self->super.worker.construct = http_dw_new;
 
   curl_global_init(CURL_GLOBAL_ALL);
 
   self->ssl_version = CURL_SSLVERSION_DEFAULT;
   self->peer_verify = TRUE;
+  /* disable batching even if the global batch_lines is specified */
+  self->super.batch_lines = 0;
+  self->batch_bytes = 0;
+  self->body_prefix = g_string_new("");
+  self->body_suffix = g_string_new("");
+  self->delimiter = g_string_new("\n");
+  self->load_balancer = http_load_balancer_new();
+  curl_version_info_data *curl_info = curl_version_info(CURLVERSION_NOW);
+  if (!self->user_agent)
+    self->user_agent = g_strdup_printf("syslog-ng %s/libcurl %s",
+                                       SYSLOG_NG_VERSION, curl_info->version);
 
   return &self->super.super.super;
 }
