@@ -43,27 +43,110 @@ import java.util.Map;
 import java.util.Map.Entry;
 import java.util.UUID;
 
+import java.util.Timer;
+import java.util.TimerTask;
+import java.util.concurrent.locks.Lock;
+import java.util.concurrent.locks.ReentrantLock;
+import static java.lang.Math.abs;
+
 public class HdfsDestination extends StructuredLogDestination {
     private static final String LOG_TAG = "HDFS:";
     private static final String HADOOP_SECURITY_AUTH_KEY = "hadoop.security.authentication";
     private static final String DFS_SUPPORT_APPEND_KEY = "dfs.support.append";
 
-    HdfsOptions options;
-    Logger logger;
-
+    private HdfsOptions options;
+    private Logger logger;
     private FileSystem hdfs;
-
     private boolean isOpened;
     private int maxFileNameLength = 255;
     private boolean appendEnabled;
+    private String archiveDir;
+    private HashMap<String, HdfsFile> openedFiles;
+    private Timer timeReap;
+    private Lock lock;
 
-    HashMap<String, HdfsFile> openedFiles;
+    class TimeReapTask extends TimerTask {
+        private boolean closeFile(HdfsFile hdfsfile) {
+           logger.info("Closing file " + hdfsfile.getPath() + "; reason=reaping");
+
+           try {
+               hdfsfile.flush();
+               hdfsfile.close();
+           } catch (IOException e) {
+               printStackTrace(e);
+               return false;
+           }
+
+           if (archiveDir != null) {
+               archiveFile(hdfsfile);
+           }
+
+           return true;
+        }
+
+        private void closeExpiredFiles() {
+           for (Iterator<Map.Entry<String, HdfsFile>> it = openedFiles.entrySet().iterator(); it.hasNext();) {
+               HdfsFile file = it.next().getValue();
+               long current = file.timeSinceLastWrite();
+
+               if (current >= options.getTimeReap()) {
+                  closeFile(file);
+                  it.remove();
+               }
+           }
+        }
+
+        private long calculateNextWakeUp() {
+           if (openedFiles.isEmpty())
+              return 0;
+
+           long nextTimerExpires = -1;
+           for (Map.Entry<String, HdfsFile> entry : openedFiles.entrySet()) {
+               logger.debug("key: " + entry.getKey());
+               long current = entry.getValue().timeSinceLastWrite();
+
+               if (current > nextTimerExpires) {
+                  nextTimerExpires = current;
+               }
+           }
+
+           if (nextTimerExpires <= 0)
+              return 0;
+
+           nextTimerExpires = Math.abs(options.getTimeReap() - nextTimerExpires);
+           logger.trace("Next schedule in " + nextTimerExpires);
+
+           return nextTimerExpires;
+        }
+
+        @Override
+        public void run() {
+           logger.debug("time-reap run");
+           lock.lock();
+           try {
+               closeExpiredFiles();
+
+               long nextTimerExpires = calculateNextWakeUp();
+
+               if (nextTimerExpires <= 0) {
+                  timeReap.cancel();
+                  timeReap = null;
+                  return;
+               }
+
+               timeReap.schedule(new TimeReapTask(), nextTimerExpires);
+           } finally {
+             lock.unlock();
+           }
+        }
+    }
 
     public HdfsDestination(long handle) {
         super(handle);
         logger = Logger.getRootLogger();
         SyslogNgInternalLogger.register(logger);
         options = new HdfsOptions(this);
+        lock = new ReentrantLock();
     }
 
     @Override
@@ -74,6 +157,13 @@ public class HdfsDestination extends StructuredLogDestination {
             logger.error(e.getMessage());
             return false;
         }
+        appendEnabled = options.getAppendEnabled();
+        archiveDir = options.getArchiveDir();
+        if(appendEnabled && archiveDir != null) {
+            logger.warn("Archiving is not allowed when append-enabled() is configured. "
+                        + "Archiving disabled.");
+            archiveDir = null;
+        }
         return true;
     }
 
@@ -85,9 +175,9 @@ public class HdfsDestination extends StructuredLogDestination {
     @Override
     public boolean open() {
         logger.debug("Opening hdfs");
+        lock.lock();
         openedFiles = new HashMap<String, HdfsFile>();
         isOpened = false;
-        appendEnabled = options.getAppendEnabled();
         try {
             Configuration configuration = new Configuration();
             loadConfigResources(configuration);
@@ -104,6 +194,8 @@ public class HdfsDestination extends StructuredLogDestination {
         } catch (Exception e) {
             printStackTrace(e);
             closeAll(false);
+        } finally {
+          lock.unlock();
         }
         return isOpened;
     }
@@ -130,45 +222,72 @@ public class HdfsDestination extends StructuredLogDestination {
         UserGroupInformation.loginUserFromKeytab(options.getKerberosPrincipal(), options.getKerberosKeytabFile());
     }
 
+
+    protected void updateFilesTimer(HdfsFile hdfsfile) {
+        if (options.getTimeReap() == 0)
+           return;
+
+        if (timeReap != null)
+           return;
+
+        timeReap = new Timer();
+        timeReap.schedule( new TimeReapTask(), options.getTimeReap());
+    }
+
     @Override
     public boolean send(LogMessage logMessage) {
         isOpened = false;
         String resolvedFileName = options.getFileNameTemplate().getResolvedString(logMessage);
-        FSDataOutputStream fsDataOutputStream = getFSDataOutputStream(resolvedFileName);
-        if (fsDataOutputStream == null) {
+        lock.lock();
+        HdfsFile hdfsfile = getHdfsFile(resolvedFileName);
+        if (hdfsfile == null) {
             // Unable to open file
-            closeAll(options.getArchiveDir() != null);
+            closeAll(true);
             return false;
         }
 
         try {
             String formattedMessage = options.getTemplate().getResolvedString(logMessage);
+
             logger.debug("Outgoing message: " + formattedMessage);
-            fsDataOutputStream.write(formattedMessage.getBytes(Charset.forName("UTF-8")));
+
+            byte[] rawMessage = formattedMessage.getBytes(Charset.forName("UTF-8"));
+
+            hdfsfile.write(rawMessage);
+
+            updateFilesTimer(hdfsfile);
         } catch (IOException e) {
             printStackTrace(e);
             closeAll(false);
             return false;
+        } finally {
+          lock.unlock();
         }
 
+
         isOpened = true;
         return true;
     }
 
-    private FSDataOutputStream getFSDataOutputStream(String resolvedFileName) {
+    private HdfsFile getHdfsFile(String resolvedFileName) {
         HdfsFile hdfsFile = openedFiles.get(resolvedFileName);
-        if (hdfsFile == null) {
+        if (hdfsFile == null || !hdfsFile.isOpen()) {
             hdfsFile = createHdfsFile(resolvedFileName);
-            openedFiles.put(resolvedFileName, hdfsFile);
+            if (hdfsFile != null)
+                openedFiles.put(resolvedFileName, hdfsFile);
         }
-        return hdfsFile.getFsDataOutputStream();
+        return hdfsFile;
     }
 
     private HdfsFile createHdfsFile(String resolvedFileName) {
-        HdfsFile hdfsFile = new HdfsFile();
+        HdfsFile hdfsFile = null;
+
         Path filePath = getFilePath(resolvedFileName);
-        hdfsFile.setPath(filePath);
-        hdfsFile.setFsDataOutputStream(createFsDataOutputStream(hdfs, filePath));
+        FSDataOutputStream fsDataOutputStream = createFsDataOutputStream(hdfs, filePath);
+
+        if (fsDataOutputStream != null)
+            hdfsFile = new HdfsFile(filePath, fsDataOutputStream);
+
         return hdfsFile;
     }
 
@@ -224,28 +343,41 @@ public class HdfsDestination extends StructuredLogDestination {
          */
 
         logger.debug("Flushing hdfs");
+        lock.lock();
+        try {
         for (Map.Entry<String, HdfsFile> entry : openedFiles.entrySet()) {
-            FSDataOutputStream outputStream = entry.getValue().getFsDataOutputStream();
-            if (outputStream != null) {
-                try {
-                    outputStream.hflush();
-                    outputStream.hsync();
-                } catch (IOException e) {
-                    logger.debug(String.format("Flush failed on file %s, reason: %s", entry.getKey(), e.getMessage()));
-                }
+            HdfsFile hdfsfile = entry.getValue();
+            try {
+                hdfsfile.flush();
+            } catch (IOException e) {
+                logger.debug(String.format("Flush failed on file %s, reason: %s", entry.getKey(), e.getMessage()));
             }
         }
+        } finally {
+          lock.unlock();
+        }
     }
 
     @Override
     public void close() {
         logger.debug("Closing hdfs");
-        closeAll(options.getArchiveDir() != null);
+
+        lock.lock();
+        try {
+        closeAll(true);
+        } finally {
+          lock.unlock();
+        }
         isOpened = false;
     }
 
     private void closeAll(boolean isArchiving) {
-        closeDataOutputStreams();
+        if (timeReap != null) {
+            timeReap.cancel();
+            timeReap = null;
+        }
+
+        closeAllHdfsFiles();
 
         if (isArchiving) {
             archiveFiles();
@@ -255,47 +387,52 @@ public class HdfsDestination extends StructuredLogDestination {
         isOpened = false;
     }
 
-    private void closeDataOutputStreams() {
+    private void closeAllHdfsFiles() {
         for (Map.Entry<String, HdfsFile> entry : openedFiles.entrySet()) {
-            FSDataOutputStream outputStream = entry.getValue().getFsDataOutputStream();
-            if (outputStream != null) {
-                try {
-                    logger.debug(String.format("Closing file: %s", entry.getKey()));
-                    outputStream.close();
-                } catch (IOException e) {
-                    printStackTrace(e);
-                }
-            }
-            entry.getValue().setFsDataOutputStream(null);
-        }
-    }
-
-    private void closeHdfs() {
-        if (hdfs != null) {
+            HdfsFile hdfsfile = entry.getValue();
             try {
-                hdfs.close();
+                logger.debug(String.format("Closing file: %s", entry.getKey()));
+                hdfsfile.flush();
+                hdfsfile.close();
             } catch (IOException e) {
-                hdfs = null;
                 printStackTrace(e);
             }
         }
     }
 
-    private void archiveFiles() {
-        if (options.getArchiveDir() != null) {
-            for (Map.Entry<String, HdfsFile> entry : openedFiles.entrySet()) {
-                Path filePath = entry.getValue().getPath();
-                logger.debug(String.format("Trying to archive %s to %s", filePath.getName(), options.getArchiveDir()));
-                Path archiveDirPath = new Path(String.format("%s/%s", options.getUri(), options.getArchiveDir()));
-                try {
-                    hdfs.mkdirs(archiveDirPath);
-                    hdfs.rename(filePath, archiveDirPath);
-                } catch (IOException e) {
-                    logger.debug(String.format("Unable to archive, reason: %s", e.getMessage()));
-                }
-            }
-        }
+    private void closeHdfs() {
+        if (hdfs == null)
+           return;
 
+        try {
+            hdfs.close();
+        } catch (IOException e) {
+            printStackTrace(e);
+        } finally {
+            hdfs = null;
+        }
+    }
+
+    private void archiveFile(HdfsFile hdfsfile) {
+        Path filePath = hdfsfile.getPath();
+        logger.debug(String.format("Trying to archive %s to %s", filePath.getName(), archiveDir));
+        Path archiveDirPath = new Path(String.format("%s/%s", options.getUri(), archiveDir));
+
+        try {
+           hdfs.mkdirs(archiveDirPath);
+           hdfs.rename(filePath, archiveDirPath);
+        } catch (IOException e) {
+            logger.debug(String.format("Unable to archive, reason: %s", e.getMessage()));
+        }
+    }
+
+    private void archiveFiles() {
+        if (archiveDir == null)
+           return;
+
+        for (Map.Entry<String, HdfsFile> entry : openedFiles.entrySet()) {
+            archiveFile( entry.getValue() );
+        }
     }
 
     private void printStackTrace(Throwable e) {
