@@ -22,17 +22,17 @@
  *
  */
 #include "python-logmsg.h"
+#include "python-helpers.h"
 #include "logmsg/logmsg.h"
 #include "messages.h"
 #include "str-utils.h"
+#include "logstamp.h"
 
-typedef struct _PyLogMessage
+int
+py_is_log_message(PyObject *obj)
 {
-  PyObject_HEAD
-  LogMessage *msg;
-} PyLogMessage;
-
-static PyTypeObject py_log_message_type;
+  return PyType_IsSubtype(Py_TYPE(obj), &py_log_message_type);
+}
 
 static int
 _str_cmp(const void *s1, const void *s2)
@@ -67,16 +67,19 @@ _is_key_blacklisted(const gchar *key)
 }
 
 static PyObject *
-_py_log_message_getattr(PyObject *o, PyObject *key)
+_py_log_message_subscript(PyObject *o, PyObject *key)
 {
-  if (!py_is_string(key))
-    return NULL;
+  if (!_py_is_string(key))
+    {
+      PyErr_SetString(PyExc_TypeError, "key is not a string object");
+      return NULL;
+    }
 
-  const gchar *name = py_object_as_string(key);
+  const gchar *name = _py_get_string_as_string(key);
 
   if (_is_key_blacklisted(name))
     {
-      msg_error("Blacklisted attribute requested", evt_tag_str("key", name));
+      PyErr_Format(PyExc_KeyError, "Blacklisted attribute %s was requested", name);
       return NULL;
     }
   NVHandle handle = log_msg_get_value_handle(name);
@@ -86,7 +89,7 @@ _py_log_message_getattr(PyObject *o, PyObject *key)
 
   if (!value)
     {
-      PyErr_SetString(PyExc_AttributeError, "No such attribute");
+      PyErr_Format(PyExc_KeyError, "No such name-value pair %s", name);
       return NULL;
     }
 
@@ -96,23 +99,41 @@ _py_log_message_getattr(PyObject *o, PyObject *key)
 }
 
 static int
-_py_log_message_setattr(PyObject *o, PyObject *key, PyObject *value)
+_py_log_message_ass_subscript(PyObject *o, PyObject *key, PyObject *value)
 {
-  if (!py_is_string(key))
-    return -1;
-
-  PyLogMessage *py_msg = (PyLogMessage *)o;
-  const gchar *name = py_object_as_string(key);
-  NVHandle handle = log_msg_get_value_handle(name);
-  PyObject *value_as_strobj = PyObject_Str(value);
-  if (value_as_strobj)
+  if (!_py_is_string(key))
     {
-      log_msg_set_value(py_msg->msg, handle, py_object_as_string(value_as_strobj), -1);
-      Py_DECREF(value_as_strobj);
+      PyErr_SetString(PyExc_TypeError, "key is not a string object");
+      return -1;
+    }
+
+  PyLogMessage *py_msg = (PyLogMessage *) o;
+  LogMessage *msg = py_msg->msg;
+  const gchar *name = _py_get_string_as_string(key);
+
+  if (log_msg_is_write_protected(msg))
+    {
+      PyErr_Format(PyExc_TypeError,
+                   "Log message is read only, cannot set name-value pair %s, "
+                   "you are possibly trying to change a LogMessage from a "
+                   "destination driver,  which is not allowed", name);
+      return -1;
+    }
+
+  NVHandle handle = log_msg_get_value_handle(name);
+
+  if (value && _py_is_string(value))
+    {
+      log_msg_set_value(py_msg->msg, handle, _py_get_string_as_string(value), -1);
     }
   else
     {
-      msg_warning("Cannot set value in logmsg", evt_tag_str("name", name));
+      PyErr_Format(PyExc_ValueError,
+                   "str or unicode object expected as log message values, got type %s (key %s). "
+                   "Earlier syslog-ng accepted any type, implicitly converting it to a string. "
+                   "With this version please convert it explicitly to a string using str()",
+                   value->ob_type->tp_name, name);
+      return -1;
     }
 
   return 0;
@@ -122,7 +143,7 @@ static void
 py_log_message_free(PyLogMessage *self)
 {
   log_msg_unref(self->msg);
-  PyObject_Del(self);
+  Py_TYPE(self)->tp_free((PyObject *) self);
 }
 
 PyObject *
@@ -138,11 +159,34 @@ py_log_message_new(LogMessage *msg)
   return (PyObject *) self;
 }
 
+static PyObject *
+py_log_message_new_empty(PyTypeObject *subtype, PyObject *args, PyObject *kwds)
+{
+  const gchar *message = NULL;
+  gint message_length = 0;
+
+  static const gchar *kwlist[] = {"message", NULL};
+  if (!PyArg_ParseTupleAndKeywords(args, kwds, "|z#", (gchar **) kwlist, &message, &message_length))
+    return NULL;
+
+  PyLogMessage *self = (PyLogMessage *) subtype->tp_alloc(subtype, 0);
+  if (!self)
+    return NULL;
+
+  self->msg = log_msg_new_empty();
+  invalidate_cached_time();
+
+  if (message)
+    log_msg_set_value(self->msg, LM_V_MESSAGE, message, message_length);
+
+  return (PyObject *) self;
+}
+
 static PyMappingMethods py_log_message_mapping =
 {
   .mp_length = NULL,
-  .mp_subscript = (binaryfunc) _py_log_message_getattr,
-  .mp_ass_subscript = (objobjargproc) _py_log_message_setattr
+  .mp_subscript = (binaryfunc) _py_log_message_subscript,
+  .mp_ass_subscript = (objobjargproc) _py_log_message_ass_subscript
 };
 
 static gboolean
@@ -150,7 +194,10 @@ _collect_nvpair_names_from_logmsg(NVHandle handle, const gchar *name, const gcha
                                   gpointer user_data)
 {
   PyObject *list = (PyObject *)user_data;
-  PyList_Append(list, PyBytes_FromString(name));
+
+  PyObject *py_name = PyBytes_FromString(name);
+  PyList_Append(list, py_name);
+  Py_XDECREF(py_name);
 
   return FALSE;
 }
@@ -189,7 +236,9 @@ _collect_macro_names(gpointer key, gpointer value, gpointer user_data)
 
   if (_is_macro_name_visible_to_user(logmsg, name))
     {
-      PyList_Append(list, PyBytes_FromString(name));
+      PyObject *py_name = PyBytes_FromString(name);
+      PyList_Append(list, py_name);
+      Py_XDECREF(py_name);
     }
 }
 
@@ -206,30 +255,99 @@ _logmessage_get_keys_method(PyLogMessage *self)
   return keys;
 }
 
+static PyObject *
+py_log_message_set_pri(PyLogMessage *self, PyObject *args, PyObject *kwrds)
+{
+  guint pri;
+
+  static const gchar *kwlist[] = {"pri", NULL};
+  if (!PyArg_ParseTupleAndKeywords(args, kwrds, "I", (gchar **) kwlist, &pri))
+    return NULL;
+
+  self->msg->pri = pri;
+
+  Py_RETURN_NONE;
+}
+
+static PyObject *
+py_log_message_set_timestamp(PyLogMessage *self, PyObject *args, PyObject *kwrds)
+{
+  PyObject *py_timestamp;
+
+  static const gchar *kwlist[] = {"timestamp", NULL};
+  if (!PyArg_ParseTupleAndKeywords(args, kwrds, "O", (gchar **) kwlist, &py_timestamp))
+    return NULL;
+
+  if (!py_datetime_to_logstamp((PyObject *) py_timestamp, &self->msg->timestamps[LM_TS_STAMP]))
+    return NULL;
+
+  Py_RETURN_NONE;
+}
+
+static PyObject *
+py_log_message_parse(PyObject *_none, PyObject *args, PyObject *kwrds)
+{
+  const gchar *raw_msg;
+  gint raw_msg_length;
+
+  PyObject *py_parse_options;
+
+  static const gchar *kwlist[] = {"raw_msg", "parse_options", NULL};
+  if (!PyArg_ParseTupleAndKeywords(args, kwrds, "s#O", (gchar **) kwlist, &raw_msg, &raw_msg_length, &py_parse_options))
+    return NULL;
+
+  if (!PyCapsule_CheckExact(py_parse_options))
+    {
+      PyErr_Format(PyExc_TypeError, "Parse options (PyCapsule) expected in the second parameter");
+      return NULL;
+    }
+
+  MsgFormatOptions *parse_options = PyCapsule_GetPointer(py_parse_options, NULL);
+  if (!parse_options)
+    {
+      PyErr_Clear();
+      PyErr_Format(PyExc_TypeError, "Invalid parse options (PyCapsule)");
+      return NULL;
+    }
+
+  PyLogMessage *py_msg = PyObject_New(PyLogMessage, &py_log_message_type);
+  if (!py_msg)
+    {
+      PyErr_Format(PyExc_TypeError, "Error creating new PyLogMessage");
+      return NULL;
+    }
+
+  py_msg->msg = log_msg_new(raw_msg, raw_msg_length, NULL, parse_options);
+
+  return (PyObject *) py_msg;
+}
+
 static PyMethodDef py_log_message_methods[] =
 {
-  {
-    "keys", (PyCFunction)_logmessage_get_keys_method,
-    METH_NOARGS, "Return keys."
-  },
+  { "keys", (PyCFunction)_logmessage_get_keys_method, METH_NOARGS, "Return keys." },
+  { "set_pri", (PyCFunction)py_log_message_set_pri, METH_VARARGS | METH_KEYWORDS, "Set priority" },
+  { "set_timestamp", (PyCFunction)py_log_message_set_timestamp, METH_VARARGS | METH_KEYWORDS, "Set timestamp" },
+  { "parse", (PyCFunction)py_log_message_parse, METH_STATIC|METH_VARARGS|METH_KEYWORDS, "Parse and create LogMessage" },
   {NULL}
 };
 
-static PyTypeObject py_log_message_type =
+PyTypeObject py_log_message_type =
 {
-  PyObject_HEAD_INIT(&PyType_Type)
+  PyVarObject_HEAD_INIT(&PyType_Type, 0)
   .tp_name = "LogMessage",
   .tp_basicsize = sizeof(PyLogMessage),
   .tp_dealloc = (destructor) py_log_message_free,
   .tp_flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,
   .tp_doc = "LogMessage class encapsulating a syslog-ng log message",
-  .tp_new = PyType_GenericNew,
+  .tp_new = py_log_message_new_empty,
   .tp_as_mapping = &py_log_message_mapping,
   .tp_methods = py_log_message_methods,
+  0,
 };
 
 void
-python_log_message_init(void)
+py_log_message_init(void)
 {
   PyType_Ready(&py_log_message_type);
+  PyModule_AddObject(PyImport_AddModule("syslogng"), "LogMessage", (PyObject *) &py_log_message_type);
 }
