@@ -36,7 +36,7 @@
 
 typedef struct
 {
-  LogThrDestDriver super;
+  LogThreadedDestDriver super;
 
   gchar *host;
   gint   port;
@@ -116,8 +116,8 @@ redis_dd_get_template_options(LogDriver *d)
  * Utilities
  */
 
-static gchar *
-redis_dd_format_stats_instance(LogThrDestDriver *d)
+static const gchar *
+redis_dd_format_stats_instance(LogThreadedDestDriver *d)
 {
   RedisDriver *self = (RedisDriver *)d;
   static gchar persist_name[1024];
@@ -171,24 +171,17 @@ authenticate_to_redis(RedisDriver *self, const gchar *password)
 }
 
 static gboolean
-redis_dd_connect(RedisDriver *self, gboolean reconnect)
+redis_dd_connect(RedisDriver *self)
 {
-  redisReply *reply;
-
-  if (reconnect && (self->c != NULL))
+  if (self->c && check_connection_to_redis(self))
     {
-      reply = redisCommand(self->c, "ping");
-
-      if (reply)
-        freeReplyObject(reply);
-
-      if (!self->c->err)
-        return TRUE;
-      else
-        self->c = redisConnect(self->host, self->port);
+      return TRUE;
     }
-  else
-    self->c = redisConnect(self->host, self->port);
+
+  if (self->c)
+    redisFree(self->c);
+
+  self->c = redisConnect(self->host, self->port);
 
   if (self->c->err)
     {
@@ -219,7 +212,7 @@ redis_dd_connect(RedisDriver *self, gboolean reconnect)
 }
 
 static void
-redis_dd_disconnect(LogThrDestDriver *s)
+redis_dd_disconnect(LogThreadedDestDriver *s)
 {
   RedisDriver *self = (RedisDriver *)s;
 
@@ -233,7 +226,7 @@ redis_dd_disconnect(LogThrDestDriver *s)
  */
 
 static worker_insert_result_t
-redis_worker_insert(LogThrDestDriver *s, LogMessage *msg)
+redis_worker_insert(LogThreadedDestDriver *s, LogMessage *msg)
 {
   RedisDriver *self = (RedisDriver *)s;
   redisReply *reply;
@@ -241,7 +234,7 @@ redis_worker_insert(LogThrDestDriver *s, LogMessage *msg)
   size_t argvlen[5];
   int argc = 2;
 
-  if (!redis_dd_connect(self, TRUE))
+  if (!redis_dd_connect(self))
     return WORKER_INSERT_RESULT_NOT_CONNECTED;
 
   if (self->c->err)
@@ -254,14 +247,14 @@ redis_worker_insert(LogThrDestDriver *s, LogMessage *msg)
     }
 
   log_template_format(self->key, msg, &self->template_options, LTZ_SEND,
-                      self->super.seq_num, NULL, self->key_str);
+                      self->super.worker.instance.seq_num, NULL, self->key_str);
 
   if (self->param1)
     log_template_format(self->param1, msg, &self->template_options, LTZ_SEND,
-                        self->super.seq_num, NULL, self->param1_str);
+                        self->super.worker.instance.seq_num, NULL, self->param1_str);
   if (self->param2)
     log_template_format(self->param2, msg, &self->template_options, LTZ_SEND,
-                        self->super.seq_num, NULL, self->param2_str);
+                        self->super.worker.instance.seq_num, NULL, self->param2_str);
 
   argv[0] = self->command->str;
   argvlen[0] = self->command->len;
@@ -309,7 +302,7 @@ redis_worker_insert(LogThrDestDriver *s, LogMessage *msg)
 }
 
 static void
-redis_worker_thread_init(LogThrDestDriver *d)
+redis_worker_thread_init(LogThreadedDestDriver *d)
 {
   RedisDriver *self = (RedisDriver *)d;
 
@@ -320,17 +313,19 @@ redis_worker_thread_init(LogThrDestDriver *d)
   self->param1_str = g_string_sized_new(1024);
   self->param2_str = g_string_sized_new(1024);
 
-  redis_dd_connect(self, FALSE);
+  redis_dd_connect(self);
 }
 
 static void
-redis_worker_thread_deinit(LogThrDestDriver *d)
+redis_worker_thread_deinit(LogThreadedDestDriver *d)
 {
   RedisDriver *self = (RedisDriver *)d;
 
   g_string_free(self->key_str, TRUE);
   g_string_free(self->param1_str, TRUE);
   g_string_free(self->param2_str, TRUE);
+
+  redis_dd_disconnect(d);
 }
 
 /*
@@ -343,7 +338,7 @@ redis_dd_init(LogPipe *s)
   RedisDriver *self = (RedisDriver *)s;
   GlobalConfig *cfg = log_pipe_get_config(s);
 
-  if (!log_dest_driver_init_method(s))
+  if (!log_threaded_dest_driver_init_method(s))
     return FALSE;
 
   if (!self->key && !self->param1)
@@ -360,7 +355,7 @@ redis_dd_init(LogPipe *s)
               evt_tag_str("host", self->host),
               evt_tag_int("port", self->port));
 
-  return log_threaded_dest_driver_start(s);
+  return log_threaded_dest_driver_start_workers(&self->super);
 }
 
 static void
@@ -401,7 +396,7 @@ redis_dd_new(GlobalConfig *cfg)
   self->super.worker.disconnect = redis_dd_disconnect;
   self->super.worker.insert = redis_worker_insert;
 
-  self->super.format.stats_instance = redis_dd_format_stats_instance;
+  self->super.format_stats_instance = redis_dd_format_stats_instance;
   self->super.stats_source = SCS_REDIS;
 
   redis_dd_set_host((LogDriver *)self, "127.0.0.1");
