@@ -26,6 +26,7 @@
 #include "messages.h"
 #include "plugin.h"
 #include "plugin-types.h"
+#include "str-utils.h"
 
 #include <fcntl.h>
 #include <sys/utsname.h>
@@ -52,6 +53,7 @@ system_sysblock_add_unix_dgram_driver(GString *sysblock, const gchar *path,
     g_string_append_printf(sysblock, " perm(%s)", perms);
   if (recvbuf_size)
     g_string_append_printf(sysblock, " so_rcvbuf(%s)", recvbuf_size);
+  g_string_append(sysblock, " flags(syslog-protocol)");
   g_string_append(sysblock, ");\n");
 }
 
@@ -180,6 +182,37 @@ _is_fd_pollable(gint fd)
   return pollable;
 }
 
+static gboolean
+_detect_linux_dev_kmsg(void)
+{
+  gint fd;
+
+  if ((fd = open("/dev/kmsg", O_RDONLY)) != -1)
+    {
+      if ((lseek (fd, 0, SEEK_END) != -1) && _is_fd_pollable(fd))
+        {
+          return TRUE;
+        }
+      close (fd);
+    }
+  return FALSE;
+}
+
+static gboolean
+_detect_linux_proc_kmsg(void)
+{
+  gint fd;
+
+  if ((fd = open("/proc/kmsg", O_RDONLY)) != -1)
+    {
+      if (_is_fd_pollable(fd))
+        {
+          return TRUE;
+        }
+      close (fd);
+    }
+  return FALSE;
+}
 
 static void
 system_sysblock_add_systemd_source(GString *sysblock)
@@ -190,55 +223,32 @@ system_sysblock_add_systemd_source(GString *sysblock)
 static void
 system_sysblock_add_linux_kmsg(GString *sysblock)
 {
-  gchar *kmsg = "/proc/kmsg";
-  int fd;
-  gchar *format = NULL;
+  const gchar *kmsg = NULL;
+  const gchar *format = NULL;
 
-  if ((fd = open("/dev/kmsg", O_RDONLY)) != -1)
+  if (_detect_linux_dev_kmsg())
     {
-      if ((lseek (fd, 0, SEEK_END) != -1) && _is_fd_pollable(fd))
-        {
-          kmsg = "/dev/kmsg";
-          format = "linux-kmsg";
-        }
-      close (fd);
+      kmsg = "/dev/kmsg";
+      format = "linux-kmsg";
     }
-
-  if (access(kmsg, R_OK) == -1)
+  else if (_detect_linux_proc_kmsg())
     {
-      msg_warning("system(): The kernel message buffer is not readable, "
-                  "please check permissions if this is unintentional.",
-                  evt_tag_str("device", kmsg),
-                  evt_tag_errno("error", errno));
+      kmsg = "/proc/kmsg";
+      format = NULL;
     }
   else
-    system_sysblock_add_file(sysblock, kmsg, -1,
-                             "kernel", "kernel", format, TRUE);
-}
-
-static gboolean
-_is_running_in_linux_container(void)
-{
-  FILE *f;
-  char line[2048];
-  gboolean container = FALSE;
-
-  f = fopen("/proc/1/cgroup", "r");
-  if (!f)
-    return FALSE;
-
-  while (fgets(line, sizeof(line), f) != NULL)
     {
-      if (line[strlen(line) - 2] != '/')
-        {
-          container = TRUE;
-          break;
-        }
+      msg_notice("system(): Neither of the Linux kernel log devices was detected, continuing without polling either /proc/kmsg or /dev/kmsg");
     }
 
-  fclose (f);
-
-  return container;
+  if (kmsg)
+    {
+      msg_debug("system(): Enabling Linux kernel log device",
+                evt_tag_str("device", kmsg),
+                evt_tag_str("format", format));
+      system_sysblock_add_file(sysblock, kmsg, -1,
+                               "kernel", "kernel", format, TRUE);
+    }
 }
 
 static void
@@ -249,20 +259,19 @@ system_sysblock_add_linux(GString *sysblock)
   else
     {
       system_sysblock_add_unix_dgram(sysblock, "/dev/log", NULL, "8192");
-      if (!_is_running_in_linux_container())
-        system_sysblock_add_linux_kmsg(sysblock);
+      system_sysblock_add_linux_kmsg(sysblock);
     }
 }
 
 static gboolean
-system_generate_system_transports(GString *sysblock)
+system_generate_system_transports(GString *sysblock, CfgArgs *args)
 {
   struct utsname u;
 
   if (uname(&u) < 0)
     {
       msg_error("system(): Cannot get information about the running kernel",
-                evt_tag_errno("error", errno));
+                evt_tag_error("error"));
       return FALSE;
     }
 
@@ -304,6 +313,10 @@ system_generate_system_transports(GString *sysblock)
     {
       system_sysblock_add_unix_dgram(sysblock, "/dev/log", NULL, NULL);
     }
+  else if (strcmp(u.sysname, "OpenBSD") == 0)
+    {
+      g_string_append(sysblock, "openbsd();");
+    }
   else
     {
       msg_error("system(): Error detecting platform, unable to define the system() source. "
@@ -337,15 +350,22 @@ system_generate_app_parser(GlobalConfig *cfg, GString *sysblock, CfgArgs *args)
 }
 
 static gboolean
-system_source_generate(CfgBlockGenerator *self, GlobalConfig *cfg, CfgArgs *args, GString *sysblock)
+system_source_generate(CfgBlockGenerator *self, GlobalConfig *cfg, CfgArgs *args, GString *sysblock,
+                       const gchar *reference)
 {
   gboolean result = FALSE;
 
+  /* NOTE: we used to have an exclude-kmsg() option that was removed, still
+   * we need to ignore it */
+
+  if (args)
+    cfg_args_remove(args, "exclude-kmsg");
+
   g_string_append(sysblock,
                   "channel {\n"
                   "    source {\n");
 
-  if (!system_generate_system_transports(sysblock))
+  if (!system_generate_system_transports(sysblock, args))
     {
       goto exit;
     }
