@@ -199,7 +199,7 @@ _set_value_in_message(JournalReaderOptions *options, LogMessage *msg, gchar *key
 }
 
 static const gchar *
-_get_value_from_message(JournalReaderOptions *options, LogMessage *msg,  gchar *key, gssize *value_length)
+_get_value_from_message(JournalReaderOptions *options, LogMessage *msg,  const gchar *key, gssize *value_length)
 {
   gchar name_with_prefix[256];
 
@@ -224,21 +224,17 @@ static void
 _set_program(JournalReaderOptions *options, LogMessage *msg)
 {
   gssize value_length = 0;
-  /* g_strdup: referred value can change during log_msg_set_value if nvtable realloc needed */
-  gchar *value = g_strdup(_get_value_from_message(options, msg, "SYSLOG_IDENTIFIER", &value_length));
+  const gchar *value_ref = _get_value_from_message(options, msg, "SYSLOG_IDENTIFIER", &value_length);
 
-  if (value_length > 0)
+  if (value_length <= 0)
     {
-      log_msg_set_value(msg, LM_V_PROGRAM, value, value_length);
-    }
-  else
-    {
-      value = g_strdup(_get_value_from_message(options, msg, "_COMM", &value_length));
-      log_msg_set_value(msg, LM_V_PROGRAM, value, value_length);
+      value_ref = _get_value_from_message(options, msg, "_COMM", &value_length);
     }
 
+  /* we need to strdup the value_ref: referred value can change during log_msg_set_value if nvtable realloc needed */
+  gchar *value = g_strdup(value_ref);
+  log_msg_set_value(msg, LM_V_PROGRAM, value, value_length);
   g_free(value);
-
 }
 
 static void
@@ -274,16 +270,19 @@ _handle_message(JournalReader *self)
   return log_source_free_to_send(&self->super);
 }
 
-static void
+static gboolean
 _alloc_state(JournalReader *self)
 {
   self->persist_handle = persist_state_alloc_entry(self->persist_state, self->persist_name, sizeof(JournalReaderState));
   JournalReaderState *state = persist_state_map_entry(self->persist_state, self->persist_handle);
+  if (!state)
+    return FALSE;
 
   state->header.version = 0;
   state->header.big_endian = (G_BYTE_ORDER == G_BIG_ENDIAN);
 
   persist_state_unmap_entry(self->persist_state, self->persist_handle);
+  return TRUE;
 }
 
 static gboolean
@@ -347,25 +346,72 @@ _skip_old_records(JournalReader *self)
   return TRUE;
 }
 
+static gboolean
+_journal_seek(Journald *journal, const gchar *cursor)
+{
+  gint rc = journald_seek_cursor(journal, cursor);
+
+  if (rc != 0)
+    {
+      msg_warning("Failed to seek journal to the saved cursor position",
+                  evt_tag_str("cursor", cursor),
+                  evt_tag_errno("error", -rc));
+      return FALSE;
+    }
+
+  return TRUE;
+}
+
+static gboolean
+_journal_next(Journald *journal)
+{
+  gint rc = journald_next(journal);
+
+  if (rc != 1)
+    {
+      msg_warning("Failed to step cursor",
+                  evt_tag_errno("error", -rc));
+      return FALSE;
+    }
+
+  return TRUE;
+}
+
+static gboolean
+_journal_test_cursor(Journald *journal, const gchar *cursor)
+{
+  gint rc = journald_test_cursor(journal, cursor);
+
+  if (rc <= 0)
+    {
+      msg_warning("Current position not matches to the saved cursor position, seek to head",
+                  evt_tag_str("cursor", cursor),
+                  evt_tag_errno("error", -rc));
+      return FALSE;
+    }
+
+  return TRUE;
+}
+
 static inline gboolean
 _seek_to_saved_state(JournalReader *self)
 {
   JournalReaderState *state = persist_state_map_entry(self->persist_state, self->persist_handle);
-  gint rc = journald_seek_cursor(self->journal, state->cursor);
-  persist_state_unmap_entry(self->persist_state, self->persist_handle);
-  if (rc != 0)
+
+  if (!_journal_seek(self->journal, state->cursor) ||
+      !_journal_next(self->journal) ||
+      !_journal_test_cursor(self->journal, state->cursor))
     {
-      msg_warning("Failed to seek journal to the saved cursor position",
-                  evt_tag_str("cursor", state->cursor),
-                  evt_tag_errno("error", -rc));
+      persist_state_unmap_entry(self->persist_state, self->persist_handle);
+
       return _seek_to_head(self);
     }
-  else
-    {
-      msg_debug("Seeking the journal to the last cursor position",
-                evt_tag_str("cursor", state->cursor));
-    }
-  journald_next(self->journal);
+
+  msg_debug("Seeking the journal to the last cursor position",
+            evt_tag_str("cursor", state->cursor));
+
+  persist_state_unmap_entry(self->persist_state, self->persist_handle);
+
   return TRUE;
 }
 
@@ -374,7 +420,11 @@ _set_starting_position(JournalReader *self)
 {
   if (!_load_state(self))
     {
-      _alloc_state(self);
+      if (!_alloc_state(self))
+        {
+          msg_error("JournalReader: Failed to allocate state");
+          return FALSE;
+        }
       if (self->super.options->read_old_records)
         return _seek_to_head(self);
       return _skip_old_records(self);
@@ -466,7 +516,6 @@ _work_finished(gpointer s)
     {
       _update_watches(self);
     }
-  log_pipe_unref(&self->super.super);
 }
 
 static void
@@ -495,7 +544,6 @@ _io_process_input(gpointer s)
   JournalReader *self = (JournalReader *) s;
 
   _stop_watches(self);
-  log_pipe_ref(&self->super.super);
   if ((self->options->flags & JR_THREADED))
     {
       main_loop_io_worker_job_submit(&self->io_job);
@@ -627,6 +675,8 @@ _init_watches(JournalReader *self)
   self->io_job.user_data = self;
   self->io_job.work = (void (*)(void *)) _work_perform;
   self->io_job.completion = (void (*)(void *)) _work_finished;
+  self->io_job.engage = (void (*)(void *)) log_pipe_ref;
+  self->io_job.release = (void (*)(void *)) log_pipe_unref;
 }
 
 JournalReader *
@@ -662,7 +712,7 @@ journal_reader_options_init(JournalReaderOptions *options, GlobalConfig *cfg, co
 
   if (options->prefix == NULL)
     {
-      gchar *default_prefix = ".journald.";
+      const gchar *default_prefix = ".journald.";
       if (cfg_is_config_version_older(cfg, VERSION_VALUE_3_8))
         {
           msg_warning("WARNING: Default value changed for the prefix() option of systemd-journal source in " VERSION_3_8,
