@@ -59,7 +59,6 @@
 #include <iv.h>
 #include <iv_signal.h>
 
-static gchar *installer_version = NULL;
 static gboolean display_version = FALSE;
 static gboolean display_module_registry = FALSE;
 static gboolean dummy = FALSE;
@@ -115,7 +114,7 @@ get_installer_version(gchar **inst_version)
           gchar *pos = strchr(line, '=');
           if (pos)
             {
-              *inst_version = strdup(pos+1);
+              *inst_version = g_strdup(pos+1);
               result = TRUE;
               break;
             }
@@ -131,9 +130,11 @@ get_installer_version(gchar **inst_version)
 void
 version(void)
 {
+  gchar *installer_version;
+
   if (!get_installer_version(&installer_version) || installer_version == NULL)
     {
-      installer_version = SYSLOG_NG_VERSION;
+      installer_version = g_strdup(SYSLOG_NG_VERSION);
     }
   printf(SYSLOG_NG_PACKAGE_NAME " " SYSLOG_NG_COMBINED_VERSION "\n"
          "Config version: " VERSION_CURRENT_VER_ONLY "\n"
@@ -147,6 +148,7 @@ version(void)
 
   printf("Module-Directory: %s\n", get_installation_path_for(SYSLOG_NG_PATH_MODULEDIR));
   printf("Module-Path: %s\n", resolvedConfigurablePaths.initial_module_path);
+  printf("Include-Path: %s\n", get_installation_path_for(SYSLOG_NG_PATH_CONFIG_INCLUDEDIR));
   printf("Available-Modules: ");
   plugin_list_modules(stdout, FALSE);
 
@@ -166,6 +168,8 @@ version(void)
          ON_OFF_STR(SYSLOG_NG_ENABLE_TCP_WRAPPER),
          ON_OFF_STR(SYSLOG_NG_ENABLE_LINUX_CAPS),
          ON_OFF_STR(SYSLOG_NG_ENABLE_SYSTEMD));
+
+  g_free(installer_version);
 }
 
 #if SYSLOG_NG_ENABLE_LINUX_CAPS
@@ -178,6 +182,9 @@ setup_caps (void)
   static gchar *capsstr_syslog = BASE_CAPS "cap_syslog=ep";
   static gchar *capsstr_sys_admin = BASE_CAPS "cap_sys_admin=ep";
 
+  if (!g_process_is_cap_enabled())
+    return;
+
   /* Set up the minimal privilege we'll need
    *
    * NOTE: polling /proc/kmsg requires cap_sys_admin, otherwise it'll always
@@ -208,8 +215,6 @@ main(int argc, char *argv[])
 
   g_process_set_argv_space(argc, (gchar **) argv);
 
-  setup_caps();
-
   resolved_configurable_paths_init(&resolvedConfigurablePaths);
 
   ctx = g_option_context_new("syslog-ng");
@@ -243,6 +248,8 @@ main(int argc, char *argv[])
       return 0;
     }
 
+  setup_caps();
+
   if(startup_debug_flag && debug_flag)
     {
       startup_debug_flag = FALSE;
