@@ -322,6 +322,7 @@ extract_holder_license_sh() {
 sed --regexp-extended "
  s~^\
 (#![^<]*<br>|)\
+(#[^<]*<br>|)\
 #############################################################################<br>\
 (\
 (# Copyright \(c\) ([0-9, -]+) [^ <][^<]*<br>)+\
@@ -338,7 +339,7 @@ sed --regexp-extended "
 #<br>\
 #############################################################################<br>\
 .*$\
-~\2\n\5~
+~\3\n\6~
 t success
  s~^.*$~~
 :success
@@ -479,6 +480,7 @@ prune_ignored_paths() {
    s~$~(/.*)?$~
   " |
   cat > "$IGNORE"
+  echo "*~" >> "$IGNORE"
  fi
 
  grep --invert-match --extended-regexp --file "$IGNORE"
