@@ -52,25 +52,10 @@ typedef struct _TimestampFormatTestCase
   gchar *expected_format;
 } TimestampFormatTestCase;
 
-__attribute__((constructor))
-static void global_test_init(void)
-{
-  app_startup();
-}
-
-__attribute__((destructor))
-static void global_test_deinit(void)
-{
-  app_shutdown();
-}
-
 void
 set_time_zone(const gchar *time_zone)
 {
-  static char envbuf[64];
-
-  snprintf(envbuf, sizeof(envbuf), "TZ=%s", time_zone);
-  putenv(envbuf);
+  setenv("TZ", time_zone, TRUE);
   tzset();
   clean_time_cache();
 }
@@ -140,6 +125,8 @@ assert_timestamp_format(GString *target, LogStamp *stamp, TimestampFormatTestCas
   cr_assert_str_eq(target->str, c.expected_format, "Actual: %s, Expected: %s", target->str, c.expected_format);
 }
 
+TestSuite(zone, .init = app_startup, .fini = app_shutdown);
+
 Test(zone, test_time_zone_offset)
 {
   TimezoneOffsetTestCase test_cases[] =
